# Copyright 2019 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from oslo_log import log
import re

from vitrage.evaluator.condition import parse_condition
from vitrage.evaluator.template_data import EdgeDescription
from vitrage.evaluator.template_data import ENTITY
from vitrage.evaluator.template_data import RELATIONSHIP
from vitrage.evaluator.template_data import Scenario
from vitrage.evaluator.template_data import TemplateData
from vitrage.evaluator.template_fields import TemplateFields as TFields
from vitrage.evaluator.template_loading.props_converter import PropsConverter
from vitrage.evaluator.template_loading.scenario_loader import \
    calculate_action_target
from vitrage.evaluator.template_loading.subgraph_builder import SubGraphBuilder
from vitrage.evaluator.template_validation.base import ValidationError
from vitrage.graph import Edge
from vitrage.graph import Vertex


LOG = log.getLogger(__name__)


class TemplateLoader(object):

    def load(self, template_schema, template_def, def_templates=None):
        template = copy.deepcopy(template_def)
        entities = _build_entities(template)
        relationships = _build_condition_relationships(template, entities)
        return TemplateData(scenarios=_build_scenarios(
            template, entities, relationships, template_schema))


def _build_entities(template):
    entities = dict()
    for template_id, entity in template[TFields.ENTITIES].items():
        properties = PropsConverter.convert_props_with_dictionary(entity)
        entities[template_id] = Vertex(template_id, properties)
    return entities


def _build_condition_relationships(template, entities):
    relationships = dict()
    for scenario in template[TFields.SCENARIOS]:
        condition = scenario.get(TFields.CONDITION)
        extracted_relationships, processed_condition = \
            _process_condition(condition, entities)
        relationships.update(extracted_relationships)
        scenario[TFields.CONDITION] = processed_condition

    return relationships


def _process_condition(condition, entities):
    """Process the condition, while extracting the condition relationships

    Example:
    condition: 'host_alarm [ on ] host AND host [contains] instance'

    regex matches:
    match group 1: 'host_alarm'
    match group 2: 'on'
    match group 3: 'host'
    ..

    Example returns:
    processed_condition: 'host_alarm__on__host AND host__contains__instance'
    extracted_relationships: {
       host_alarm__on__host: EdgeDescription(...),
       host__contains__instance: EdgeDescription(...),
       }
    """
    regex = r"(\w+)\s*\[\s*(\w+)\s*\]\s*(\w+)"
    extracted_relationships = dict()

    def relation_str(matchobj):
        source = matchobj.group(1)
        label = matchobj.group(2)
        target = matchobj.group(3)
        relation_id = '%s__%s__%s' % (source, label, target)
        extracted_relationships[relation_id] = EdgeDescription(
            Edge(source, target, label, dict()),
            entities[source],
            entities[target])
        return relation_id

    processed_condition = re.sub(regex, relation_str, condition)
    return extracted_relationships, processed_condition


def _build_scenarios(template, entities, relationships, schema):
    name = template[TFields.METADATA][TFields.NAME]

    scenarios = []
    for index, scenario in enumerate(template[TFields.SCENARIOS]):
        s_id = "%s-scenario%d" % (name, index)
        condition = parse_condition(scenario.get(TFields.CONDITION))
        default_target = calculate_action_target(condition,
                                                 entities,
                                                 relationships)

        if not default_target:
            raise ValidationError(135, 'scenario %d' % index,
                                  scenario.get(TFields.CONDITION))

        tmp_scenario = Scenario(
            id=s_id,
            version=schema.version(),
            condition=None,
            actions=_build_actions(schema, scenario, s_id, default_target),
            subgraphs=_build_subgraphs(condition, entities, relationships),
            entities=entities,
            relationships=relationships)
        scenarios.append(tmp_scenario)
    return scenarios


def _build_actions(template_schema, scenario, scenario_id, default_target):
    actions = []
    actions_def = scenario[TFields.ACTIONS]
    for counter, action_def in enumerate(actions_def):
        action_id = '%s-action%d' % (scenario_id, counter)
        action_type, action_props = action_def.popitem()
        action = template_schema.loaders.get(action_type).load(
            action_id,
            default_target,
            action_props,
            action_type
        )
        actions.append(action)
    return actions


def _build_subgraphs(condition, entities, relationships):

    def _extract_var(symbol_name):
        if symbol_name in relationships:
            edge_descriptor = relationships[symbol_name]
            return edge_descriptor, RELATIONSHIP
        elif symbol_name in entities:
            vertex = entities[symbol_name]
            return vertex, ENTITY

    return SubGraphBuilder.from_condition(condition, _extract_var)



# Copyright 2019 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from testtools import matchers

from vitrage.api_handler.apis.template import TemplateApis
from vitrage.tests.functional.test_configuration import TestConfiguration
from vitrage.tests.mocks import utils
from vitrage.tests.unit.entity_graph.base import TestEntityGraphUnitBase


class TestTemplates(TestEntityGraphUnitBase, TestConfiguration):

    VALIDATION_FAILED = 'validation failed'
    VALIDATION_OK = 'validation OK'

    class MockNotifier(object):
        def notify(self, event_type, data):
            pass

    def setUp(self):
        super(TestTemplates, self).setUp()
        self.add_db()
        self.apis = TemplateApis(notifier=self.MockNotifier(), db=self._db)
        self.added_template = None

    def tearDown(self):
        super(TestTemplates, self).tearDown()
        self._delete_templates()

    def _load_template_content(self, template_filename):
        template_path = '%s/templates/parameters/%s' % (
            utils.get_resources_dir(),
            template_filename)
        return [(template_path, self._load_yaml_file(template_path))]

    def _validate_template_with_no_params(self, template_filename):
        files_content = self._load_template_content(template_filename)
        # Action
        results = self.apis.validate_template(
            ctx=None, templates=files_content, template_type=None, params=None)

        # Test assertions
        self._assert_validate_template_result(
            self.VALIDATION_FAILED, 163,
            'Failed to resolve parameter', results)

    def _validate_template_with_missing_param(self, template_filename):
        # Setup
        apis = TemplateApis(notifier=self.MockNotifier(), db=self._db)

        files_content = self._load_template_content(template_filename)
        params = {'template_name': 'template_with_params_1',
                  'alarm_name': 'My alarm', 'new_state': 'SUBOPTIMAL'}

        # Action
        results = apis.validate_template(ctx=None, templates=files_content,
                                         template_type=None, params=params)

        # Test assertions
        self._assert_validate_template_result(
            self.VALIDATION_FAILED, 163,
            'Failed to resolve parameter', results)

    def _validate_template_with_actual_params(self, template_filename):
        # Setup
        apis = TemplateApis(notifier=self.MockNotifier(), db=self._db)
        files_content = self._load_template_content(template_filename)
        params = {'template_name': 'template_with_params_2',
                  'alarm_type': 'zabbix', 'alarm_name': 'My alarm',
                  'new_state': 'SUBOPTIMAL'}

        # Action
        results = apis.validate_template(ctx=None, templates=files_content,
                                         template_type=None, params=params)

        # Test assertions
        self._assert_validate_template_result(
            self.VALIDATION_OK, 0, 'Template validation is OK', results)

    def _validate_template_with_missing_param_def(self, template_filename):
        # Setup
        apis = TemplateApis(notifier=self.MockNotifier(), db=self._db)
        files_content = self._load_template_content(template_filename)
        params = {'alarm_type': 'zabbix', 'alarm_name': 'My alarm',
                  'new_state': 'SUBOPTIMAL'}

        # Action
        results = apis.validate_template(ctx=None, templates=files_content,
                                         template_type=None, params=params)

        # Test assertions
        self._assert_validate_template_result(
            self.VALIDATION_FAILED, 161, 'get_param called for a parameter '
            'that is not defined in the \'parameters\' block', results)

    def _validate_template_without_params(self, template_filename):
        # Setup
        apis = TemplateApis(notifier=self.MockNotifier(), db=self._db)
        files_content = self._load_template_content(template_filename)

        # Action
        results = apis.validate_template(ctx=None, templates=files_content,
                                         template_type=None, params=None)

        # Test assertions
        self._assert_validate_template_result(
            self.VALIDATION_OK, 0, 'Template validation is OK', results)

    def _validate_template_with_extra_actual_param(self, template_filename):
        # Setup
        apis = TemplateApis(notifier=self.MockNotifier(), db=self._db)
        files_content = self._load_template_content(template_filename)
        params = {'template_name': 'template_with_params_2',
                  'alarm_type': 'zabbix', 'alarm_name': 'My alarm',
                  'new_state': 'SUBOPTIMAL',
                  'non_existing_param': 'some value'}

        # Action
        results = apis.validate_template(ctx=None, templates=files_content,
                                         template_type=None, params=params)

        # Test assertions
        self._assert_validate_template_result(
            self.VALIDATION_OK, 0, 'Template validation is OK', results)

    def _validate_template_with_extra_param_def(self, template_filename):
        # Setup
        apis = TemplateApis(notifier=self.MockNotifier(), db=self._db)
        files_content = self._load_template_content(template_filename)
        params = {'template_name': 'template_with_params_2',
                  'alarm_type': 'zabbix', 'alarm_name': 'My alarm',
                  'new_state': 'SUBOPTIMAL'}

        # Action
        results = apis.validate_template(ctx=None, templates=files_content,
                                         template_type=None, params=params)

        # Test assertions
        self._assert_validate_template_result(
            self.VALIDATION_OK, 0, 'Template validation is OK', results)

    def _add_template_with_no_params(self, template_filename):
        # Setup
        files_content = self._load_template_content(template_filename)

        # Action.
        added_templates = \
            self.apis.add_template(ctx=None, templates=files_content,
                                   template_type=None, params=None)
        self.added_template = added_templates[0]['uuid']

        # Test assertions
        self.assertThat(added_templates, matchers.HasLength(1))
        self.assertEqual('ERROR', added_templates[0]['status'])
        self.assert_starts_with('Failed to resolve parameter',
                                added_templates[0]['status details'])

    def _add_template_with_missing_param(self, template_filename):
        # Setup
        files_content = self._load_template_content(template_filename)
        params = {'template_name': 'template_with_params_3',
                  'alarm_name': 'My alarm', 'new_state': 'SUBOPTIMAL'}

        # Action
        added_templates = \
            self.apis.add_template(ctx=None, templates=files_content,
                                   template_type=None, params=params)
        self.added_template = added_templates[0]['uuid']

        # Test assertions
        self.assertThat(added_templates, matchers.HasLength(1))
        self.assertEqual('ERROR', added_templates[0]['status'])
        self.assert_starts_with('Failed to resolve parameter',
                                added_templates[0]['status details'])

    def _add_template_with_actual_params(self, template_filename):
        # Setup
        files_content = self._load_template_content(template_filename)
        params = {'template_name': 'template_with_params_4',
                  'alarm_type': 'zabbix', 'alarm_name': 'My alarm',
                  'new_state': 'SUBOPTIMAL'}

        # Action
        added_templates = \
            self.apis.add_template(ctx=None, templates=files_content,
                                   template_type=None, params=params)
        self.added_template = added_templates[0]['uuid']

        # Test assertions
        self.assertThat(added_templates, matchers.HasLength(1))
        self.assertEqual('LOADING', added_templates[0]['status'])

    def _add_template_with_missing_param_def(self, template_filename):
        # Setup
        files_content = self._load_template_content(template_filename)
        params = {'alarm_type': 'zabbix', 'alarm_name': 'My alarm',
                  'new_state': 'SUBOPTIMAL'}

        # Action
        added_templates = \
            self.apis.add_template(ctx=None, templates=files_content,
                                   template_type=None, params=params)
        self.added_template = added_templates[0]['uuid']

        # Test assertions
        self.assertEqual('ERROR', added_templates[0]['status'])
        self.assert_starts_with('get_param called for a parameter that is not '
                                'defined in the \'parameters\' block',
                                added_templates[0]['status details'])

    def _add_template_without_params(self, template_filename):
        # Setup
        files_content = self._load_template_content(template_filename)

        # Action
        added_templates = \
            self.apis.add_template(ctx=None, templates=files_content,
                                   template_type=None, params=None)
        self.added_template = added_templates[0]['uuid']

        # Test assertions
        self.assertThat(added_templates, matchers.HasLength(1))
        self.assertEqual('LOADING', added_templates[0]['status'])

    def _add_template_with_extra_actual_param(self, template_filename):
        # Setup
        files_content = self._load_template_content(template_filename)
        params = {'template_name': 'template_with_extra_actual_param',
                  'alarm_type': 'zabbix', 'alarm_name': 'My alarm',
                  'new_state': 'SUBOPTIMAL',
                  'non_existing_param': 'some value'}

        # Action
        added_templates = \
            self.apis.add_template(ctx=None, templates=files_content,
                                   template_type=None, params=params)
        self.added_template = added_templates[0]['uuid']

        # Test assertions
        self.assertThat(added_templates, matchers.HasLength(1))
        self.assertEqual('LOADING', added_templates[0]['status'])

    def _add_template_with_extra_param_def(self, template_filename):
        # Setup
        files_content = self._load_template_content(template_filename)
        params = {'template_name': 'template_with_extra_param_def',
                  'alarm_type': 'zabbix', 'alarm_name': 'My alarm',
                  'new_state': 'SUBOPTIMAL'}

        # Action
        added_templates = \
            self.apis.add_template(ctx=None, templates=files_content,
                                   template_type=None, params=params)
        self.added_template = added_templates[0]['uuid']

        # Test assertions
        self.assertThat(added_templates, matchers.HasLength(1))
        self.assertEqual('LOADING', added_templates[0]['status'])

    def _assert_validate_template_result(self, expected_status,
                                         expected_status_code,
                                         expected_message, results):
        self.assertIsNotNone(results)
        results = json.loads(results)
        results = results['results']
        self.assertIsNotNone(results)
        self.assertThat(results, matchers.HasLength(1))
        self.assertEqual(expected_status, results[0]['status'])
        self.assertEqual(expected_status_code, results[0]['status code'])
        self.assert_starts_with(expected_message, results[0]['message'])

    def _delete_templates(self):
        if self.added_template:
            self.apis.delete_template(ctx=None, uuids=[self.added_template])



# Copyright 2015 - Alcatel-Lucent
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""Functions for generating transformer-output events """

import random

import vitrage.tests.mocks.trace_generator as tg


def generate_random_events_list(generator_spec_list):
    """Generates random events for the generators given.

     Each element in the list of generators includes a generator and
     number of events to generate for it's entities

     :param generator_spec_list: list of generators
     :type generator_spec_list: list

     :return: list of datasource events
     :rtype list

    """

    data = []
    for spec in generator_spec_list:
        generator = spec[tg.GENERATOR]
        data += tg.generate_data_stream(generator.models, spec[tg.NUM_EVENTS])
    random.shuffle(data)
    return data


def simple_instance_generators(host_num, vm_num, snapshot_events=0,
                               snap_vals=None):
    """A simple function for returning vm generators.

    Returns generators for a given number of hosts and
    instances. Instances will be distributed across hosts in round-robin style.

    :param host_num: number of hosts
    :param vm_num: number of vms
    :param snapshot_events: number of snapshot events per instance
    :param snap_vals: number of update events per instance
    :return: generators for vm_num vms as specified
    """

    mapping = [('vm-{0}'.format(ind), 'host-{0}'.format(ind % host_num))
               for ind in range(vm_num)
               ]

    test_entity_spec_list = [
        {tg.DYNAMIC_INFO_FKEY: tg.TRANS_INST_SNAPSHOT_D,
         tg.STATIC_INFO_FKEY: tg.TRANS_INST_SNAPSHOT_S,
         tg.MAPPING_KEY: mapping,
         tg.EXTERNAL_INFO_KEY: snap_vals,
         tg.NAME_KEY: 'Instance (vm) snapshot generator',
         tg.NUM_EVENTS: snapshot_events
         }
    ]

    return tg.get_trace_generators(test_entity_spec_list)


def simple_host_generators(zone_num, host_num, snapshot_events=0,
                           snap_vals=None):
    """A simple function for returning vm generators.

    Returns generators for a given number of hosts and
    instances. Instances will be distributed across hosts in round-robin style.

    :param zone_num: number of hosts
    :param host_num: number of vms
    :param snapshot_events: number of snapshot events per instance
    :param snap_vals: number of update events per instance
    :return: generators for vm_num vms as specified
    """

    mapping = [('host-{0}'.format(ind), 'zone-{0}'.format(ind % zone_num))
               for ind in range(host_num)
               ]

    test_entity_spec_list = [
        {tg.DYNAMIC_INFO_FKEY: tg.TRANS_HOST_SNAPSHOT_D,
         tg.STATIC_INFO_FKEY: tg.TRANS_HOST_SNAPSHOT_S,
         tg.MAPPING_KEY: mapping,
         tg.EXTERNAL_INFO_KEY: snap_vals,
         tg.NAME_KEY: 'Host snapshot generator',
         tg.NUM_EVENTS: snapshot_events
         }
    ]

    return tg.get_trace_generators(test_entity_spec_list)


def simple_zone_generators(zone_num, snapshot_events=0, snap_vals=None):
    """A simple function for returning vm generators.

    Returns generators for a given number of hosts and
    instances. Instances will be distributed across hosts in round-robin style.

    :param zone_num: number of hosts
    :param snapshot_events: number of snapshot events per instance
    :param snap_vals: number of update events per instance
    :return: generators for vm_num vms as specified
    """

    mapping = [('zone-{0}'.format(ind), 'cluster-0')
               for ind in range(zone_num)]

    test_entity_spec_list = [
        {tg.DYNAMIC_INFO_FKEY: tg.TRANS_ZONE_SNAPSHOT_D,
         tg.STATIC_INFO_FKEY: tg.TRANS_ZONE_SNAPSHOT_S,
         tg.MAPPING_KEY: mapping,
         tg.EXTERNAL_INFO_KEY: snap_vals,
         tg.NAME_KEY: 'Zone snapshot generator',
         tg.NUM_EVENTS: snapshot_events
         }
    ]
    return tg.get_trace_generators(test_entity_spec_list)


def simple_aodh_alarm_generators(alarm_num,
                                 snapshot_events=0, snap_vals=None):
    """A simple function for returning aodh alarm generators.

    Returns generators for a given number of alarms.

    :param alarm_num: number of alarms
    :param snapshot_events: number of snapshot events
    :param snap_vals: values of snapshot
    :return: generators for alarm_num alarms as specified
    """

    mapping = [('alarm-{0}'.format(ind), 'resource-{0}'.format(ind))
               for ind in range(alarm_num)
               ]

    test_entity_spec_list = [
        {tg.DYNAMIC_INFO_FKEY: tg.TRANS_AODH_SNAPSHOT_D,
         tg.DYNAMIC_INFO_FPATH: tg.MOCK_TRANSFORMER_PATH,
         tg.STATIC_INFO_FKEY: None,
         tg.MAPPING_KEY: mapping,
         tg.EXTERNAL_INFO_KEY: snap_vals,
         tg.NAME_KEY: 'Aodh snapshot generator',
         tg.NUM_EVENTS: snapshot_events
         }
    ]
    return tg.get_trace_generators(test_entity_spec_list)


def simple_aodh_update_alarm_generators(alarm_num,
                                        update_events=0,
                                        update_vals=None):
    """A simple function for returning aodh alarm generators.

    Returns generators for a given number of alarms.

    :param alarm_num: number of alarms
    :param update_events: number of update events
    :param update_vals: values of update
    :return: generators for alarm_num alarms as specified
    """

    mapping = [('alarm-{0}'.format(ind), 'resource-{0}'.format(ind))
               for ind in range(alarm_num)
               ]

    test_entity_spec_list = [
        {tg.DYNAMIC_INFO_FKEY: tg.TRANS_AODH_UPDATE_D,
         tg.DYNAMIC_INFO_FPATH: tg.MOCK_TRANSFORMER_PATH,
         tg.STATIC_INFO_FKEY: None,
         tg.MAPPING_KEY: mapping,
         tg.EXTERNAL_INFO_KEY: update_vals,
         tg.NAME_KEY: 'Aodh update generator',
         tg.NUM_EVENTS: update_events
         }
    ]
    return tg.get_trace_generators(test_entity_spec_list)


def simple_doctor_alarm_generators(update_vals=None):
    """A function for returning Doctor alarm event generators.

    Returns generators for a given number of Doctor alarms.

    :param update_vals: preset values for ALL update events
    :return: generators for alarms as specified
    """
    return _simple_alarm_generators('Doctor',
                                    tg.TRANS_DOCTOR_UPDATE_D, update_vals)


def simple_collectd_alarm_generators(update_vals=None):
    """A function for returning Collectd alarm event generators.

    Returns generators for a given number of Collectd alarms.

    :param update_vals: preset values for ALL update events
    :return: generators for alarms as specified
    """
    return _simple_alarm_generators('Collectd',
                                    tg.TRANS_COLLECTD_UPDATE_D, update_vals)


def simple_kapacitor_alarm_generators(update_vals=None):
    """A function for returning Kapacitor alarm event generators.

    Returns generators for a given number of Kapacitor alarms.

    :param update_vals: preset values for ALL update events
    :return: generators for alarms as specified
    """
    return _simple_alarm_generators('Kapacitor',
                                    tg.TRANS_KAPACITOR_UPDATE_D, update_vals)


def simple_prometheus_alarm_generators(update_vals=None):
    """A function for returning Prometheus alert event generators.

    Returns generators for a given number of Prometheus alerts.

    :param update_vals: preset values for ALL update events
    :return: generators for alarms as specified
    """
    return _simple_alarm_generators('Prometheus',
                                    tg.TRANS_PROMETHEUS_UPDATE_D, update_vals)


def _simple_alarm_generators(datasource, sample_file, update_vals):
    """A function for returning alarm event generators.

    Returns generators for a given number of alarms.

    :param update_vals: preset values for ALL update events
    :return: generators for alarms as specified
    """

    test_entity_spec_list = [({
        tg.DYNAMIC_INFO_FKEY: sample_file,
        tg.DYNAMIC_INFO_FPATH: tg.MOCK_TRANSFORMER_PATH,
        tg.STATIC_INFO_FKEY: None,
        tg.EXTERNAL_INFO_KEY: update_vals,
        tg.MAPPING_KEY: None,
        tg.NAME_KEY: datasource + ' alarm generator',
        tg.NUM_EVENTS: 1
    })]

    return tg.get_trace_generators(test_entity_spec_list)



#!/usr/bin/env python

# coding: utf-8

# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
from datetime import datetime
import logging
from logging.handlers import RotatingFileHandler
from oslo_config import cfg
import oslo_messaging as messaging
from oslo_utils import uuidutils
import socket


'''
Expected input:
Send To:
Vitrage Message Bus address e.g.
rabbit://userrabbit:passrabbit@127.0.0.1:5672/
Subject: {TRIGGER.STATUS}
Message:
    host={HOST.NAME1}
    hostid={HOST.ID1}
    hostip={HOST.IP1}
    triggerid={TRIGGER.ID}
    description={TRIGGER.NAME}
    rawtext={TRIGGER.NAME.ORIG}
    expression={TRIGGER.EXPRESSION}
    value={TRIGGER.VALUE}
    priority={TRIGGER.NSEVERITY}
    lastchange={EVENT.DATE} {EVENT.TIME}
'''


LOG_FILE = '/var/log/zabbix/zabbix_vitrage.log'
LOG_MAX_SIZE = 10000000
LOG_FORMAT = '%(asctime)s.%(msecs).03d %(name)s[%(process)d]' \
             ' %(threadName)s %(levelname)s - %(message)s'
LOG_DATE_FMT = '%Y.%m.%d %H:%M:%S'
ZABBIX_EVENT_TYPE = 'zabbix.alarm'

debug = False


def create_payload(body):
    payload = {}
    for line in body.splitlines():
        key, prop = line.split('=', 1)
        payload[key.lower().strip()] = prop.strip()
    return payload


def main():

    parser = argparse.ArgumentParser()
    parser.add_argument('sendto', help='Vitrage message bus path')
    parser.add_argument('topic', help='zabbix topic')
    parser.add_argument('body', help='zabbix body')
    args = parser.parse_args()

    logging.info('SENDTO: %s', args.sendto)
    logging.info('TOPIC: %s', args.topic)
    logging.info('BODY:\n----\n%s\n', args.body)

    transport_url = args.sendto
    transport = messaging.get_notification_transport(cfg.CONF, transport_url)
    driver = 'messagingv2'
    publisher = 'zabbix_%s' % socket.gethostname()
    notifier = messaging.Notifier(transport,
                                  driver=driver,
                                  publisher_id=publisher,
                                  topics=['vitrage_notifications'])

    alarm_status = args.topic.lower()
    event_type = '%s.%s' % (ZABBIX_EVENT_TYPE, alarm_status)
    payload = create_payload(args.body)

    logging.info('PUBLISHER: %s', publisher)
    logging.info('EVENT_TYPE: %s', event_type)
    logging.info('\nPAYLOAD:\n%s', payload)
    notifier.info(ctxt={'message_id': uuidutils.generate_uuid(),
                        'publisher_id': publisher,
                        'timestamp': datetime.utcnow()},
                  event_type=event_type,
                  payload=payload)
    logging.info('MESSAGE SENT..')


if __name__ == '__main__':

    log = logging.getLogger()

    if debug:
        log.setLevel(logging.DEBUG)
    else:
        log.setLevel(logging.INFO)

    handler = RotatingFileHandler(filename=LOG_FILE,
                                  maxBytes=LOG_MAX_SIZE,
                                  backupCount=3)
    fmt = logging.Formatter(LOG_FORMAT, LOG_DATE_FMT)
    handler.setFormatter(fmt)
    log.addHandler(handler)

    logging.info('***----------Script start-----------***')
    try:
        main()
    except Exception as e:
        logging.exception('MESSAGE WAS NOT SENT - %s' % e)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg

from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE
from vitrage.datasources.zabbix.driver import ZabbixDriver
from vitrage.datasources.zabbix.properties import ZabbixProperties \
    as ZabbixProps
from vitrage.datasources.zabbix import ZABBIX_DATASOURCE
from vitrage.tests import base
from vitrage.tests.mocks import utils


class TestZabbixConfig(base.BaseTest):

    OPTS = [
        cfg.StrOpt(DSOpts.TRANSFORMER,
                   default='vitrage.datasources.zabbix.transformer.'
                           'ZabbixTransformer',
                   help='Zabbix data source transformer class path',
                   required=True),
        cfg.StrOpt(DSOpts.DRIVER,
                   default='vitrage.datasources.zabbix.driver.ZabbixDriver',
                   help='Zabbix driver class path',
                   required=True),
        cfg.IntOpt(DSOpts.CHANGES_INTERVAL,
                   default=30,
                   min=30,
                   help='interval between checking changes in zabbix plugin',
                   required=True),
        cfg.StrOpt('user', default='admin',
                   help='Zabbix user name'),
        cfg.StrOpt('password', default='zabbix',
                   help='Zabbix user password'),
        cfg.StrOpt('url', default='', help='Zabbix url'),
        cfg.StrOpt(DSOpts.CONFIG_FILE,
                   help='Zabbix configuration file',
                   default=utils.get_resources_dir()
                        + '/zabbix/zabbix_conf.yaml'),
    ]

    # the mappings match the ones in zabbix_conf.yaml
    MAPPINGS = {
        'compute-1': {ZabbixProps.RESOURCE_TYPE: NOVA_HOST_DATASOURCE,
                      ZabbixProps.RESOURCE_NAME: 'compute-1'},
        'compute-2': {ZabbixProps.RESOURCE_TYPE: NOVA_HOST_DATASOURCE,
                      ZabbixProps.RESOURCE_NAME: 'host2'},
    }

    NON_EXISTING_MAPPINGS = {
        'X': {ZabbixProps.RESOURCE_TYPE: NOVA_HOST_DATASOURCE,
              ZabbixProps.RESOURCE_NAME: 'compute-1'},
        'compute-1': {ZabbixProps.RESOURCE_TYPE: 'X',
                      ZabbixProps.RESOURCE_NAME: 'compute-1'},
    }

    def setUp(self):
        super(TestZabbixConfig, self).setUp()
        self.conf_reregister_opts(self.OPTS, group=ZABBIX_DATASOURCE)

    def test_zabbix_configuration_loading(self):
        # Action
        mappings = ZabbixDriver._configuration_mapping()

        # Test assertions
        self.assertEqual(len(self.MAPPINGS), len(mappings))

        for expected_mapping in self.MAPPINGS.items():
            self.assertTrue(self._check_contains(expected_mapping, mappings))
        for expected_mapping in self.NON_EXISTING_MAPPINGS.items():
            self.assertFalse(self._check_contains(expected_mapping, mappings))

    @staticmethod
    def _check_contains(expected_mapping, mappings):
        for mapping in mappings.items():
            if expected_mapping == mapping:
                return True
        return False



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from copy import deepcopy
from vitrage.evaluator.actions.recipes.execute_mistral import INPUT
from vitrage.evaluator.actions.recipes.execute_mistral import WORKFLOW
from vitrage.evaluator.template_fields import TemplateFields as TFields
from vitrage.evaluator.template_loading.v1.action_loader import \
    BaseActionLoader


class ExecuteMistralLoader(BaseActionLoader):
    def _get_properties(self, action_dict):
        """Place all properties under an 'input' block"""
        properties = action_dict.get(TFields.PROPERTIES, {})
        input_properties = deepcopy(properties)
        input_properties.pop(WORKFLOW)
        return {WORKFLOW: properties[WORKFLOW], INPUT: input_properties}



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.common.constants import DatasourceAction as DSAction
from vitrage.common.constants import GraphAction
from vitrage.common.constants import VertexProperties as VProps
from vitrage.entity_graph.mappings.operational_resource_state import \
    OperationalResourceState
from vitrage.tests.functional.base import TestFunctionalBase


class TestDatasourceInfoMapperFunctional(TestFunctionalBase):

    def setUp(self):
        super(TestDatasourceInfoMapperFunctional, self).setUp()
        self.load_datasources()

    def test_state_on_update(self):
        # setup
        processor = self.create_processor_and_graph()
        event = self._create_event(spec_type='INSTANCE_SPEC',
                                   datasource_action=DSAction.INIT_SNAPSHOT)

        # action
        processor.process_event(event)

        # test assertions
        entity = processor.transformer_manager.transform(event)
        vertex = processor.entity_graph.get_vertex(entity.vertex.vertex_id)
        self.assertEqual('ACTIVE', vertex[VProps.VITRAGE_AGGREGATED_STATE])
        self.assertEqual(OperationalResourceState.OK,
                         vertex[VProps.VITRAGE_OPERATIONAL_STATE])

    def test_state_on_neighbor_update(self):
        # setup
        vertex, neighbors, processor = self._create_entity(
            spec_type='INSTANCE_SPEC',
            datasource_action=DSAction.INIT_SNAPSHOT)
        self.assertEqual(2, processor.entity_graph.num_vertices())

        neighbors[0].vertex[VProps.STATE] = 'available'
        neighbors[0].vertex[VProps.VITRAGE_IS_PLACEHOLDER] = False

        # action
        processor._connect_neighbors(neighbors, [], GraphAction.UPDATE_ENTITY)

        # test assertions
        neighbor_vertex = processor.entity_graph.get_vertex(
            neighbors[0].vertex.vertex_id)
        self.assertEqual('AVAILABLE',
                         neighbor_vertex[VProps.VITRAGE_AGGREGATED_STATE])
        self.assertEqual(OperationalResourceState.OK,
                         neighbor_vertex[VProps.VITRAGE_OPERATIONAL_STATE])



# Copyright 2018 Samsung Electronics
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import GraphAction
from vitrage.datasources.driver_base import DriverBase
from vitrage import os_clients


class TroveDriverBase(DriverBase):

    def __init__(self):
        super(TroveDriverBase, self).__init__()
        self.__client = None
        self.__cached_entities = []

    @property
    def client(self):
        if not self.__client:
            self.__client = os_clients.trove_client()
        return self.__client

    def get_all(self, datasource_action):
        return self.make_pickleable(self._get_and_cache_all_entities(),
                                    self._get_vitrage_type(),
                                    datasource_action,
                                    *self.properties_to_filter_out())

    def get_changes(self, datasource_action):
        return self.make_pickleable(self._get_changed_entities(),
                                    self._get_vitrage_type(),
                                    datasource_action,
                                    *self.properties_to_filter_out())

    def _get_and_cache_all_entities(self):
        self.__cached_entities = self._get_all_entities()
        return self.__cached_entities

    def _get_changed_entities(self):
        actual_entities = self._get_all_entities()
        changed_entities = []

        for actual_entity in actual_entities:
            cached_entity = self._find_entity(actual_entity,
                                              self.__cached_entities)
            if cached_entity:
                # Add modified entities
                if not self._equal_entities(actual_entity, cached_entity):
                    changed_entities.append(actual_entity)
            else:
                # Add new entities
                changed_entities.append(actual_entity)

        # Delete removed entities
        for cached_entity in self.__cached_entities:
            if not self._find_entity(cached_entity, actual_entities):
                cached_entity[DSProps.EVENT_TYPE] = GraphAction.DELETE_ENTITY
                changed_entities.append(cached_entity)

        self.__cached_entities = actual_entities
        return changed_entities

    @abc.abstractmethod
    def _get_vitrage_type(self):
        pass

    @abc.abstractmethod
    def _get_all_entities(self):
        pass

    @abc.abstractmethod
    def _find_entity(self, search_entity, entities):
        pass

    @abc.abstractmethod
    def _equal_entities(self, old_entity, new_entity):
        pass

    @staticmethod
    def properties_to_filter_out():
        return ['manager', '_info']

    @staticmethod
    def should_delete_outdated_entities():
        return True

    @staticmethod
    def extract_entities(entities):
        return [entity.to_dict() for entity in entities]



# Copyright 2016 - Alcatel-Lucent
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging

from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EdgeLabel
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE
from vitrage.datasources.nova.properties import HostProperties as HostProps
from vitrage.datasources.nova.zone import NOVA_ZONE_DATASOURCE
from vitrage.datasources.resource_transformer_base import \
    ResourceTransformerBase
from vitrage.datasources import transformer_base
from vitrage.datasources.transformer_base import extract_field_value
import vitrage.graph.utils as graph_utils


LOG = logging.getLogger(__name__)


class HostTransformer(ResourceTransformerBase):

    def _create_snapshot_entity_vertex(self, entity_event):
        return self._create_vertex(entity_event,
                                   entity_event.get(HostProps.HOST))

    def _create_update_entity_vertex(self, entity_event):
        LOG.warning('Host Update is not supported yet')

    def _create_vertex(self, entity_event, host_name):

        metadata = {VProps.NAME: host_name}
        entity_key = self._create_entity_key(entity_event)

        vitrage_sample_timestamp = entity_event[DSProps.SAMPLE_DATE]
        update_timestamp = \
            self._format_update_timestamp(None,
                                          vitrage_sample_timestamp)

        return graph_utils.create_vertex(
            entity_key,
            vitrage_category=EntityCategory.RESOURCE,
            vitrage_type=NOVA_HOST_DATASOURCE,
            vitrage_sample_timestamp=vitrage_sample_timestamp,
            entity_id=host_name,
            update_timestamp=update_timestamp,
            metadata=metadata)

    def _create_snapshot_neighbors(self, entity_event):
        return self._create_host_neighbors(entity_event)

    def _create_update_neighbors(self, entity_event):
        return self._create_host_neighbors(entity_event)

    def _create_host_neighbors(self, entity_event):
        zone_name = extract_field_value(entity_event, HostProps.ZONE)
        zone_neighbor = self._create_neighbor(entity_event,
                                              zone_name,
                                              NOVA_ZONE_DATASOURCE,
                                              EdgeLabel.CONTAINS,
                                              is_entity_source=False)
        return [zone_neighbor]

    def _create_entity_key(self, entity_event):
        key_fields = self._key_values(NOVA_HOST_DATASOURCE,
                                      entity_event.get(HostProps.HOST))
        return transformer_base.build_key(key_fields)

    def get_vitrage_type(self):
        return NOVA_HOST_DATASOURCE



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from testtools.matchers import HasLength
from vitrage.evaluator.actions.base import ActionType
from vitrage.evaluator.actions.recipes.action_steps import EXECUTE_EXTERNAL
from vitrage.evaluator.actions.recipes.action_steps import EXECUTION_ENGINE
from vitrage.evaluator.actions.recipes.execute_mistral import ExecuteMistral
from vitrage.evaluator.actions.recipes.execute_mistral import MISTRAL
from vitrage.evaluator.actions.recipes.execute_mistral import WORKFLOW
from vitrage.evaluator.template_data import ActionSpecs
from vitrage.tests.base import BaseTest
from vitrage.tests.base import IsEmpty


class RaiseAlarmRecipeTest(BaseTest):

    # noinspection PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(RaiseAlarmRecipeTest, cls).setUpClass()
        cls.props = {EXECUTION_ENGINE: MISTRAL,
                     WORKFLOW: 'wf_4',
                     'host': 'host5',
                     'state': 'ok'}
        cls.action_spec = ActionSpecs(0,
                                      ActionType.EXECUTE_MISTRAL,
                                      {},
                                      cls.props)

    def test_get_do_recipe(self):

        # Test Action
        action_steps = ExecuteMistral.get_do_recipe(self.action_spec)

        # Test Assertions

        # expecting for one step: [execute_external]
        self.assertThat(action_steps, HasLength(1))

        self.assertEqual(EXECUTE_EXTERNAL, action_steps[0].type)
        execute_external_step_params = action_steps[0].params
        self.assertIsNotNone(execute_external_step_params)
        self.assertLessEqual(2, len(execute_external_step_params))

        execution_engine = execute_external_step_params[EXECUTION_ENGINE]
        self.assertEqual(self.props[EXECUTION_ENGINE], execution_engine)

        workflow = execute_external_step_params[WORKFLOW]
        self.assertEqual(self.props[WORKFLOW], workflow)

    def test_get_undo_recipe(self):

        # Test Action
        action_steps = ExecuteMistral.get_undo_recipe(self.action_spec)

        # Test Assertions

        # expecting for zero steps (no undo for this action)
        self.assertThat(action_steps, IsEmpty())



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from testtools import matchers

from vitrage.common.constants import EntityCategory
from vitrage.common.constants import TemplateTypes as TType
from vitrage.common.constants import VertexProperties as VProps
from vitrage.evaluator.scenario_repository import ScenarioRepository
from vitrage.evaluator.template_validation.template_syntax_validator import \
    syntax_validation
from vitrage.graph import Vertex
from vitrage.tests import base
from vitrage.tests.base import IsEmpty
from vitrage.tests.functional.test_configuration import TestConfiguration
from vitrage.tests.mocks import utils
from vitrage.utils import file as file_utils


class ScenarioRepositoryTest(base.BaseTest, TestConfiguration):
    def setUp(self):
        super(ScenarioRepositoryTest, self).setUp()
        self.add_db()
        templates_dir = utils.get_resources_dir() + '/templates/general'
        self.add_templates(templates_dir)
        templates_dir_path = templates_dir
        self.template_defs = file_utils.load_yaml_files(templates_dir_path)
        self.scenario_repository = ScenarioRepository()

    def test_template_loader(self):

        # Test Action
        scenario_repository = ScenarioRepository()

        # Test assertions
        self.assertIsNotNone(scenario_repository)
        self.assertThat(scenario_repository.templates,
                        matchers.HasLength(2),
                        'scenario_repository.templates '
                        'should contain all valid templates')

    def test_init_scenario_repository(self):

        # Test Setup
        valid_template_counter = 0
        for template_definition in self.template_defs:
            syntax_validation_result = syntax_validation(template_definition)
            if syntax_validation_result.is_valid_config:
                valid_template_counter += 1

        # Test assertions
        self.assertIsNotNone(self.scenario_repository)

        scenario_templates = self.scenario_repository.templates
        # there is one bad template
        self.assertThat(scenario_templates,
                        matchers.HasLength(valid_template_counter),
                        'scenario_repository.templates '
                        'should contain all valid templates')

        entity_equivalences = self.scenario_repository.entity_equivalences
        for entity_props, equivalence in entity_equivalences.items():
            # Example structure of entity_equivalences
            #   { A: (A, B, C),
            #     B: (A, B, C),
            #     C: (A, B, C)}
            # Verify entity itself is also included. It is not required, but
            # worth noting when handling equivalence
            self.assertTrue(entity_props in equivalence)
            for equivalent_props in equivalence:
                # Verify equivalent scenarios are present in repository
                self.assertTrue(equivalent_props in
                                self.scenario_repository.entity_scenarios)

    def test_get_scenario_by_edge(self):
        pass

    def test_get_scenario_by_entity(self):
        pass

    def test_add_template(self):
        pass


class RegExTemplateTest(base.BaseTest, TestConfiguration):

    def setUp(self):
        super(RegExTemplateTest, self).setUp()
        templates_dir = utils.get_resources_dir() + '/templates/regex'
        self.add_db()
        self.add_templates(templates_dir)
        self.scenario_repository = ScenarioRepository()

    def test_basic_regex(self):

        event_properties = {
            "time": 121354,
            "vitrage_type": "zabbix",
            "vitrage_category": "ALARM",
            "rawtext": "Interface virtual-0 down on {HOST.NAME}",
            "host": "some_host_kukoo"
        }
        event_vertex = Vertex(vertex_id="test_vertex",
                              properties=event_properties)
        relevant_scenarios = \
            self.scenario_repository.get_scenarios_by_vertex(
                event_vertex)
        self.assertThat(relevant_scenarios, matchers.HasLength(1))
        relevant_scenario = relevant_scenarios[0]
        self.assertEqual("zabbix_alarm_pass", relevant_scenario[0].vertex_id)

    def test_regex_with_exact_match(self):

        event_properties = {
            "time": 121354,
            "vitrage_type": "zabbix",
            "vitrage_category": "ALARM",
            "rawtext": "Public interface host43 down",
            "host": "some_host_kukoo"
        }
        event_vertex = Vertex(vertex_id="test_vertex",
                              properties=event_properties)
        relevant_scenarios = \
            self.scenario_repository.get_scenarios_by_vertex(
                event_vertex)
        self.assertThat(relevant_scenarios, matchers.HasLength(1))
        relevant_scenario = relevant_scenarios[0]
        self.assertEqual("exact_match", relevant_scenario[0].vertex_id)

    def test_basic_regex_with_no_match(self):

        event_properties = {
            "time": 121354,
            "vitrage_type": "zabbix",
            "vitrage_category": "ALARM",
            "rawtext": "No Match",
            "host": "some_host_kukoo"
        }
        event_vertex = Vertex(vertex_id="test_vertex",
                              properties=event_properties)
        relevant_scenarios = \
            self.scenario_repository.get_scenarios_by_vertex(
                event_vertex)
        self.assertThat(relevant_scenarios, IsEmpty())


class EquivalentScenarioTest(base.BaseTest, TestConfiguration):
    def setUp(self):
        super(EquivalentScenarioTest, self).setUp()
        templates_dir = utils.get_resources_dir() + \
            '/templates/equivalent_scenarios/'
        equivalences_dir = templates_dir + '/equivalences'
        def_templates_dir = utils.get_resources_dir() + \
            '/templates/def_template_tests'
        self.add_db()
        self.add_templates(templates_dir)
        self.add_templates(equivalences_dir,
                           TType.EQUIVALENCE)
        self.add_templates(def_templates_dir,
                           TType.DEFINITION)
        self.scenario_repository = ScenarioRepository()

    def test_expansion(self):
        entity_scenarios = self.scenario_repository.entity_scenarios
        for key, scenarios in entity_scenarios.items():
            if (VProps.VITRAGE_CATEGORY, EntityCategory.ALARM) in key:
                # scenarios expanded on the other alarm
                self.assertThat(scenarios, matchers.HasLength(2))
            if (VProps.VITRAGE_CATEGORY, EntityCategory.RESOURCE) in key:
                # Scenarios expanded on the two alarms. Each alarm is expanded
                # to two equivalent alarms. Thus 2 x 2 = 4 in total
                self.assertThat(scenarios, matchers.HasLength(4))
        # each relationship is expand to two. Thus 2 x 2 = 4 in total
        relationships = self.scenario_repository.relationship_scenarios.keys()
        self.assertThat(relationships, matchers.HasLength(4))



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_log import log as logging
from testtools import matchers

from vitrage.common.constants import DatasourceAction
from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EdgeLabel
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import GraphAction
from vitrage.common.constants import UpdateMethod
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.alarm_properties import AlarmProperties as AlarmProps
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE
from vitrage.datasources.nova.host.transformer import HostTransformer
from vitrage.datasources import transformer_base as tbase
from vitrage.datasources.transformer_base import TransformerBase
from vitrage.datasources.zabbix.properties import ZabbixProperties\
    as ZabbixProps
from vitrage.datasources.zabbix.properties import ZabbixTriggerSeverity
from vitrage.datasources.zabbix.properties import ZabbixTriggerValue
from vitrage.datasources.zabbix.transformer import ZabbixTransformer
from vitrage.datasources.zabbix import ZABBIX_DATASOURCE
from vitrage.tests import base
from vitrage.tests.mocks import mock_driver as mock_sync
from vitrage.utils.datetime import format_unix_timestamp

LOG = logging.getLogger(__name__)


# noinspection PyProtectedMember
class ZabbixTransformerTest(base.BaseTest):

    OPTS = [
        cfg.StrOpt(DSOpts.UPDATE_METHOD,
                   default=UpdateMethod.PULL),
    ]

    # noinspection PyAttributeOutsideInit,PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(ZabbixTransformerTest, cls).setUpClass()
        cls.transformers = {}
        cls.conf = cfg.ConfigOpts()
        cls.conf.register_opts(cls.OPTS, group=ZABBIX_DATASOURCE)
        cls.transformers[NOVA_HOST_DATASOURCE] = \
            HostTransformer(cls.transformers)

    def test_create_entity_key(self):
        LOG.debug('Test get key from nova instance transformer')

        # Test setup
        spec_list = mock_sync.simple_zabbix_alarm_generators(host_num=1,
                                                             events_num=1)
        zabbix_alarms = mock_sync.generate_sequential_events_list(spec_list)
        transformer = ZabbixTransformer(self.transformers)
        event = zabbix_alarms[0]
        self.enrich_event(event)

        # Test action
        observed_key = transformer._create_entity_key(event)

        # Test assertions
        observed_key_fields = observed_key.split(
            TransformerBase.KEY_SEPARATOR)

        self.assertEqual(EntityCategory.ALARM, observed_key_fields[0])
        self.assertEqual(event[DSProps.ENTITY_TYPE], observed_key_fields[1])
        self.assertEqual(event[ZabbixProps.RESOURCE_NAME],
                         observed_key_fields[2])
        self.assertEqual(event[ZabbixProps.TRIGGER_ID],
                         observed_key_fields[3])

    def test_zabbix_alarm_transform(self):
        LOG.debug('Zabbix alarm transformer test: transform entity event')

        # Test setup
        spec_list = mock_sync.simple_zabbix_alarm_generators(host_num=4,
                                                             events_num=10)
        zabbix_alarms = mock_sync.generate_sequential_events_list(spec_list)

        for alarm in zabbix_alarms:
            # Test action
            self.enrich_event(alarm, format_timestamp=False)
            wrapper = ZabbixTransformer(self.transformers)\
                .transform(alarm)
            self._validate_vertex(wrapper.vertex, alarm)

            neighbors = wrapper.neighbors
            self.assertThat(neighbors, matchers.HasLength(1))
            neighbor = neighbors[0]

            # Right now we are support only host as a resource
            if neighbor.vertex[VProps.VITRAGE_TYPE] == NOVA_HOST_DATASOURCE:
                self._validate_host_neighbor(neighbors[0], alarm)

            self._validate_action(alarm, wrapper)

    def _validate_action(self, alarm, wrapper):
        ds_action = alarm[DSProps.DATASOURCE_ACTION]
        if ds_action in (DatasourceAction.SNAPSHOT, DatasourceAction.UPDATE):
            if alarm[ZabbixProps.VALUE] == ZabbixTriggerValue.OK:
                self.assertEqual(GraphAction.DELETE_ENTITY, wrapper.action)
            else:
                self.assertEqual(GraphAction.UPDATE_ENTITY, wrapper.action)
        else:
            self.assertEqual(GraphAction.CREATE_ENTITY, wrapper.action)

    def _validate_vertex(self, vertex, event):

        self.assertEqual(EntityCategory.ALARM, vertex[VProps.VITRAGE_CATEGORY])
        self.assertEqual(event[DSProps.ENTITY_TYPE],
                         vertex[VProps.VITRAGE_TYPE])
        self.assertEqual(event[ZabbixProps.DESCRIPTION],
                         vertex[VProps.NAME])

        event_status = event[ZabbixProps.VALUE]

        if event_status == ZabbixTriggerValue.OK:
            self.assertEqual(AlarmProps.INACTIVE_STATE,
                             vertex[VProps.STATE])
        else:
            self.assertEqual(AlarmProps.ACTIVE_STATE,
                             vertex[VProps.STATE])

        event_severity = ZabbixTriggerSeverity.str(
            event[ZabbixProps.PRIORITY])
        self.assertEqual(event_severity, vertex[VProps.SEVERITY])

        self.assertFalse(vertex[VProps.VITRAGE_IS_DELETED])
        self.assertFalse(vertex[VProps.VITRAGE_IS_PLACEHOLDER])

    def _validate_host_neighbor(self, neighbor, event):

        host_vertex = neighbor.vertex

        observed_key = host_vertex.vertex_id

        expected_key = tbase.build_key((EntityCategory.RESOURCE,
                                        NOVA_HOST_DATASOURCE,
                                        event[ZabbixProps.RESOURCE_NAME]))
        expected_uuid = \
            TransformerBase.uuid_from_deprecated_vitrage_id(expected_key)

        self.assertEqual(expected_uuid, observed_key)
        self.assertEqual(expected_uuid,
                         host_vertex.properties.get(VProps.VITRAGE_ID))

        self.assertFalse(host_vertex[VProps.VITRAGE_IS_DELETED])
        self.assertTrue(host_vertex[VProps.VITRAGE_IS_PLACEHOLDER])

        self.assertEqual(EntityCategory.RESOURCE,
                         host_vertex[VProps.VITRAGE_CATEGORY])
        self.assertEqual(event[ZabbixProps.RESOURCE_NAME],
                         host_vertex[VProps.ID])
        self.assertEqual(NOVA_HOST_DATASOURCE,
                         host_vertex[VProps.VITRAGE_TYPE])

        edge = neighbor.edge
        self.assertEqual(EdgeLabel.ON, edge.label)

        alarm_key = ZabbixTransformer(self.transformers).\
            _create_entity_key(event)
        alarm_uuid = TransformerBase.uuid_from_deprecated_vitrage_id(alarm_key)
        self.assertEqual(alarm_uuid, edge.source_id)
        self.assertEqual(host_vertex.vertex_id, edge.target_id)

    @staticmethod
    def enrich_event(event, format_timestamp=True):
        if format_timestamp:
            event[ZabbixProps.TIMESTAMP] = format_unix_timestamp(
                event[ZabbixProps.LAST_CHANGE], tbase.TIMESTAMP_FORMAT)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg


OPTS = [
    cfg.IntOpt('min_time_to_delete',
               default=60,
               min=60,
               help='minimum time until deleting entity (in seconds)'),
    cfg.IntOpt('initialization_interval',
               default=1,
               min=1,
               deprecated_for_removal=True,
               deprecated_since='Stein',
               deprecated_reason='This config option is no longer used.',
               help='interval between consistency initialization checks for '
                    'finding if all end messages from datasources were '
                    'received (in seconds)'),
    cfg.IntOpt('initialization_max_retries',
               default=30,
               min=1,
               deprecated_for_removal=True,
               deprecated_since='Stein',
               deprecated_reason='This config option is no longer used.',
               help='maximum retries for consistency initialization '
                    'for finding if all end messages from datasources were '
                    'received (in seconds)'),
]



# Copyright 2016 - Alcatel-Lucent
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from oslo_log import log

from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import VertexProperties as VProps
from vitrage.utils import datetime as datetime_utils

LOG = log.getLogger(__name__)


class DriverBase(object, metaclass=abc.ABCMeta):

    _datasource_name = None

    @abc.abstractmethod
    def get_all(self, datasource_action):
        pass

    def callback_on_fault(self, exception):
        pass

    def get_changes(self, datasource_action):
        pass

    @classmethod
    def make_pickleable(cls, entities, entity_type, datasource_action, *args):
        return cls.make_pickleable_without_end_msg(
            entities, entity_type, datasource_action, *args)

    @classmethod
    def make_pickleable_without_end_msg(cls, entities, entity_type,
                                        datasource_action, *args):
        pickleable_entities = []
        for entity in entities:
            for field in args:
                entity.pop(field, None)

            cls._add_entity_type(entity, entity_type)
            cls._add_datasource_action(entity, datasource_action)
            cls._add_sampling_time(entity)
            entity[VProps.VITRAGE_DATASOURCE_NAME] = cls._datasource_name
            pickleable_entities.append(entity)
        return pickleable_entities

    @classmethod
    def make_pickleable_iter(cls, entities, entity_type,
                             datasource_action, *args):
        for entity in entities:
            for field in args:
                entity.pop(field, None)

            cls._add_entity_type(entity, entity_type)
            cls._add_datasource_action(entity, datasource_action)
            cls._add_sampling_time(entity)
            entity[VProps.VITRAGE_DATASOURCE_NAME] = cls._datasource_name
            yield entity

    @staticmethod
    def _add_entity_type(entity, entity_type):
        if DSProps.ENTITY_TYPE not in entity:
            entity[DSProps.ENTITY_TYPE] = entity_type

    @staticmethod
    def _add_sampling_time(entity):
        entity[DSProps.SAMPLE_DATE] = datetime_utils.format_utcnow()

    @staticmethod
    def _add_datasource_action(entity, datasource_action):
        entity[DSProps.DATASOURCE_ACTION] = datasource_action

    def enrich_event(self, event, event_type):
        """Return the given event with extra fields

        We add extra data, which the transformer uses later on.
        For example, we can add a timestamp and change the message's structure
        :param event: the event received by oslo
        :param event_type: the event type of the event
        :return: the enriched event
        """

        pass

    @staticmethod
    def get_event_types():
        """Return a list of all event types relevant to this datasource

        Example:
        return ['compute.instance.update',
                'compute.instance.resume']

        It also supports prefixes- the event types which start
        with this prefix will be processed by this driver:
        Example:
        return ['compute.instance']



        :return: a list of event types
        :rtype: list of str
        """

        return []

    @staticmethod
    def properties_to_filter_out():
        """Return a list of properties to be removed from the event"""
        return []

    @staticmethod
    def should_delete_outdated_entities():
        """Should the processor delete entities when become outdated

        An entity that was not updated in the last get_all is considered
        outdated. If this method returns true, then it will be automatically
        deleted when outdated.
        Note that this behavior does not suit all datasources - datasources
        that are based only on notifications do not update their entities in
        get_all, so they should return False.
        """
        return False



# Copyright 2016 - Nokia Corporation
# Copyright 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from vitrage.tests.functional import api

VERSIONS_RESPONSE = {'versions': [
    {'id': 'v1.0',
     'links': [
         {'href': 'http://localhost/v1/',
          'rel': 'self'}],
     'status': 'CURRENT',
     'updated': '2015-11-29'}]}


class TestVersions(api.FunctionalTest):

    def __init__(self, *args, **kwds):
        super(TestVersions, self).__init__(*args, **kwds)
        self.auth = 'keystone'

    def test_versions(self):
        data = self.get_json('/')
        self.assert_dict_equal(VERSIONS_RESPONSE, data)



# Copyright 2015 - Alcatel-Lucent
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg


# Register options for the service
OPTS = [
    cfg.PortOpt('port', default=8999,
                help='The port for the vitrage API server.',),
    cfg.StrOpt('host', default='0.0.0.0',
               help='The listen IP for the vitrage API server.',),
    cfg.StrOpt('paste_config', default='api-paste.ini',
               help='Configuration file for WSGI definition of API.'),
    cfg.IntOpt('workers', default=1, min=1,
               help='Number of workers for vitrage API server.'),
    cfg.StrOpt('auth_mode', default='keystone', choices={'keystone',
                                                         'noauth',
                                                         'keycloak'},
               help='Authentication mode to use.'),
    cfg.BoolOpt('check_backend', default=True,
                help='Check that backend is ready before an api call')
]



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import threading

from testtools import matchers

from vitrage.api_handler.apis.alarm import AlarmApis
from vitrage.api_handler.apis.rca import RcaApis
from vitrage.api_handler.apis.resource import ResourceApis
from vitrage.api_handler.apis.topology import TopologyApis
from vitrage.common.constants import EdgeLabel
from vitrage.common.constants import EdgeProperties
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import VertexProperties as VProps
from vitrage.common.utils import decompress_obj
from vitrage.datasources import NOVA_HOST_DATASOURCE
from vitrage.datasources import NOVA_INSTANCE_DATASOURCE
from vitrage.datasources import NOVA_ZONE_DATASOURCE
from vitrage.datasources import OPENSTACK_CLUSTER
from vitrage.datasources.transformer_base \
    import create_cluster_placeholder_vertex
from vitrage.entity_graph.mappings.operational_alarm_severity import \
    OperationalAlarmSeverity
from vitrage.entity_graph.processor.notifier import PersistNotifier
from vitrage.graph.driver.networkx_graph import edge_copy
from vitrage.graph.driver.networkx_graph import NXGraph
import vitrage.graph.utils as graph_utils
from vitrage.persistency.service import VitragePersistorEndpoint
from vitrage.tests.base import IsEmpty
from vitrage.tests.functional.test_configuration import TestConfiguration
from vitrage.tests.unit.entity_graph.base import TestEntityGraphUnitBase
from vitrage.utils.datetime import utcnow


class TestApis(TestEntityGraphUnitBase, TestConfiguration):

    def setUp(self):
        super(TestApis, self).setUp()
        self.add_db()
        self.api_lock = threading.RLock()

    def test_get_alarms_with_admin_project(self):
        # Setup
        graph = self._create_graph()
        apis = AlarmApis(graph, self.api_lock, self._db)
        ctx = {'project_id': 'project_1', 'is_admin': True}

        # Action
        alarms = apis.get_alarms(ctx, vitrage_id='all', all_tenants=False)
        alarms = decompress_obj(alarms)['alarms']

        # Test assertions
        self.assertThat(alarms, matchers.HasLength(3))
        self._check_projects_entities(alarms, 'project_1', True)

    def test_get_alarms_with_not_admin_project(self):
        # Setup
        graph = self._create_graph()
        apis = AlarmApis(graph, self.api_lock, self._db)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        alarms = apis.get_alarms(ctx, vitrage_id='all', all_tenants=False)
        alarms = decompress_obj(alarms)['alarms']

        # Test assertions
        self.assertThat(alarms, matchers.HasLength(2))
        self._check_projects_entities(alarms, 'project_2', True)

    def test_get_alarm_counts_with_not_admin_project(self):
        # Setup
        graph = self._create_graph()
        apis = AlarmApis(graph, self.api_lock, self._db)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        counts = apis.get_alarm_counts(ctx, all_tenants=False)
        counts = json.loads(counts)

        # Test assertions
        self.assertEqual(1, counts['WARNING'])
        self.assertEqual(0, counts['SEVERE'])
        self.assertEqual(1, counts['CRITICAL'])
        self.assertEqual(0, counts['OK'])
        self.assertEqual(0, counts['N/A'])

    def test_get_alarms_with_all_tenants(self):
        # Setup
        graph = self._create_graph()
        apis = AlarmApis(graph, self.api_lock, self._db)
        ctx = {'project_id': 'project_1', 'is_admin': False}

        # Action
        alarms = apis.get_alarms(ctx, vitrage_id='all', all_tenants=True)
        alarms = decompress_obj(alarms)['alarms']

        # Test assertions
        self.assertThat(alarms, matchers.HasLength(5))
        self._check_projects_entities(alarms, None, True)

    def test_get_alarm_counts_with_all_tenants(self):
        # Setup
        graph = self._create_graph()
        apis = AlarmApis(graph, self.api_lock, self._db)
        ctx = {'project_id': 'project_1', 'is_admin': False}

        # Action
        counts = apis.get_alarm_counts(ctx, all_tenants=True)
        counts = json.loads(counts)

        # Test assertions
        self.assertEqual(2, counts['WARNING'])
        self.assertEqual(2, counts['SEVERE'])
        self.assertEqual(1, counts['CRITICAL'])
        self.assertEqual(0, counts['OK'])
        self.assertEqual(0, counts['N/A'])

    def test_get_rca_with_admin_project(self):
        # Setup
        graph = self._create_graph()
        apis = RcaApis(graph, self.api_lock, self._db)
        ctx = {'project_id': 'project_1', 'is_admin': True}

        # Action
        graph_rca = apis.get_rca(ctx, root='alarm_on_host', all_tenants=False)
        graph_rca = json.loads(graph_rca)

        # Test assertions
        self.assertThat(graph_rca['nodes'], matchers.HasLength(3))
        self._check_projects_entities(graph_rca['nodes'], 'project_1', True)

    def test_get_rca_with_not_admin_project(self):
        # Setup
        graph = self._create_graph()
        apis = RcaApis(graph, self.api_lock, self._db)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        graph_rca = apis.get_rca(ctx,
                                 root='alarm_on_instance_3',
                                 all_tenants=False)
        graph_rca = json.loads(graph_rca)

        # Test assertions
        self.assertThat(graph_rca['nodes'], matchers.HasLength(2))
        self._check_projects_entities(graph_rca['nodes'], 'project_2', True)

    def test_get_rca_with_not_admin_bla_project(self):
        # Setup
        graph = self._create_graph()
        apis = RcaApis(graph, self.api_lock, self._db)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        graph_rca = apis.get_rca(ctx, root='alarm_on_host', all_tenants=False)
        graph_rca = json.loads(graph_rca)

        # Test assertions
        self.assertThat(graph_rca['nodes'], matchers.HasLength(3))
        self._check_projects_entities(graph_rca['nodes'], 'project_2', True)

    def test_get_rca_with_all_tenants(self):
        # Setup
        graph = self._create_graph()
        apis = RcaApis(graph, self.api_lock, self._db)
        ctx = {'project_id': 'project_1', 'is_admin': False}

        # Action
        graph_rca = apis.get_rca(ctx, root='alarm_on_host', all_tenants=True)
        graph_rca = json.loads(graph_rca)

        # Test assertions
        self.assertThat(graph_rca['nodes'], matchers.HasLength(5))
        self._check_projects_entities(graph_rca['nodes'], None, True)

    def test_get_topology_with_admin_project(self):
        # Setup
        graph = self._create_graph()
        apis = TopologyApis(graph, self.api_lock)
        ctx = {'project_id': 'project_1', 'is_admin': True}

        # Action
        graph_topology = apis.get_topology(
            ctx,
            graph_type='graph',
            depth=10,
            query=None,
            root=None,
            all_tenants=False)
        graph_topology = decompress_obj(graph_topology)

        # Test assertions
        self.assertThat(graph_topology['nodes'], matchers.HasLength(8))
        self._check_projects_entities(graph_topology['nodes'],
                                      'project_1',
                                      False)

    def test_get_topology_with_not_admin_project(self):
        # Setup
        graph = self._create_graph()
        apis = TopologyApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        graph_topology = apis.get_topology(
            ctx,
            graph_type='graph',
            depth=10,
            query=None,
            root=None,
            all_tenants=False)
        graph_topology = decompress_obj(graph_topology)

        # Test assertions
        self.assertThat(graph_topology['nodes'], matchers.HasLength(7))
        self._check_projects_entities(graph_topology['nodes'],
                                      'project_2',
                                      False)

    def test_get_topology_with_all_tenants(self):
        # Setup
        graph = self._create_graph()
        apis = TopologyApis(graph, self.api_lock)
        ctx = {'project_id': 'project_1', 'is_admin': False}

        # Action
        graph_topology = apis.get_topology(
            ctx,
            graph_type='graph',
            depth=10,
            query=None,
            root=None,
            all_tenants=True)
        graph_topology = decompress_obj(graph_topology)

        # Test assertions
        self.assertThat(graph_topology['nodes'], matchers.HasLength(12))

    def test_resource_list_with_admin_project(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': True}

        # Action
        resources = apis.get_resources(
            ctx,
            resource_type=None,
            all_tenants=False)
        resources = decompress_obj(resources)['resources']

        # Test assertions
        self.assertThat(resources, matchers.HasLength(5))

    def test_resource_list_with_admin_project_and_query(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': True}

        # Action
        resources = apis.get_resources(
            ctx,
            resource_type=NOVA_INSTANCE_DATASOURCE,
            all_tenants=False,
            query={'==': {'id': 'instance_3'}})
        resources = decompress_obj(resources)['resources']

        # Test assertions
        self.assertThat(resources, matchers.HasLength(1))

    def test_resource_list_with_not_admin_project(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        resources = apis.get_resources(
            ctx,
            resource_type=None,
            all_tenants=False)
        resources = decompress_obj(resources)['resources']

        # Test assertions
        self.assertThat(resources, matchers.HasLength(2))

    def test_resource_list_with_not_admin_project_and_no_existing_type(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        resources = apis.get_resources(
            ctx,
            resource_type=NOVA_HOST_DATASOURCE,
            all_tenants=False)
        resources = decompress_obj(resources)['resources']

        # Test assertions
        self.assertThat(resources, IsEmpty())

    def test_resource_list_with_not_admin_project_and_existing_type(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        resources = apis.get_resources(
            ctx,
            resource_type=NOVA_INSTANCE_DATASOURCE,
            all_tenants=False)
        resources = decompress_obj(resources)['resources']

        # Test assertions
        self.assertThat(resources, matchers.HasLength(2))

    def test_resource_list_with_all_tenants(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_1', 'is_admin': False}

        # Action
        resources = apis.get_resources(
            ctx,
            resource_type=None,
            all_tenants=True)
        resources = decompress_obj(resources)['resources']

        # Test assertions
        self.assertThat(resources, matchers.HasLength(7))

    def test_resource_count_with_admin_project(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': True}

        # Action
        resources = apis.count_resources(
            ctx,
            resource_type=None,
            all_tenants=False)
        resources = json.loads(resources)

        # Test assertions
        self.assertEqual(2, resources[NOVA_INSTANCE_DATASOURCE])
        self.assertEqual(1, resources[NOVA_ZONE_DATASOURCE])
        self.assertEqual(1, resources[OPENSTACK_CLUSTER])
        self.assertEqual(1, resources[NOVA_HOST_DATASOURCE])

    def test_resource_count_with_admin_project_and_query(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': True}

        # Action
        resources = apis.count_resources(
            ctx,
            resource_type=NOVA_INSTANCE_DATASOURCE,
            all_tenants=False,
            query={'==': {'id': 'instance_3'}})
        resources = json.loads(resources)

        # Test assertions
        self.assertEqual(1, resources[NOVA_INSTANCE_DATASOURCE])

    def test_resource_count_with_not_admin_project(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        resources = apis.count_resources(
            ctx,
            resource_type=None,
            all_tenants=False)
        resources = json.loads(resources)

        # Test assertions
        self.assertEqual(2, resources[NOVA_INSTANCE_DATASOURCE])

    def test_resource_count_with_not_admin_project_and_no_existing_type(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        resources = apis.count_resources(
            ctx,
            resource_type=NOVA_HOST_DATASOURCE,
            all_tenants=False)
        resources = json.loads(resources)

        # Test assertions
        self.assertThat(resources.items(), matchers.HasLength(0))

    def test_resource_count_with_not_admin_project_and_existing_type(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        resources = apis.count_resources(
            ctx,
            resource_type=NOVA_INSTANCE_DATASOURCE,
            all_tenants=False)
        resources = json.loads(resources)

        # Test assertions
        self.assertEqual(2, resources[NOVA_INSTANCE_DATASOURCE])

    def test_resource_count_with_all_tenants(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_1', 'is_admin': False}

        # Action
        resources = apis.count_resources(
            ctx,
            resource_type=None,
            all_tenants=True)
        resources = json.loads(resources)

        # Test assertions
        self.assertEqual(4, resources[NOVA_INSTANCE_DATASOURCE])
        self.assertEqual(1, resources[NOVA_ZONE_DATASOURCE])
        self.assertEqual(1, resources[OPENSTACK_CLUSTER])
        self.assertEqual(1, resources[NOVA_HOST_DATASOURCE])

    def test_resource_count_with_all_tenants_and_group_by(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_1', 'is_admin': False}

        # Action
        resources = apis.count_resources(
            ctx,
            resource_type=None,
            all_tenants=True,
            group_by=VProps.PROJECT_ID)
        resources = json.loads(resources)

        # Test assertions
        self.assertEqual(2, resources['project_1'])
        self.assertEqual(2, resources['project_2'])
        self.assertEqual(3, resources[''])

    def test_resource_show_with_admin_and_no_project_resource(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_1', 'is_admin': True}

        # Action
        resource = apis.show_resource(ctx, 'zone_1')
        resource = json.loads(resource)

        # Test assertions
        self.assertIsNotNone(resource)
        self._check_resource_properties(resource, 'zone_1',
                                        NOVA_ZONE_DATASOURCE)

    def test_resource_show_with_not_admin_and_no_project_resource(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_1', 'is_admin': False}

        # Action
        resource = apis.show_resource(ctx, 'zone_1')

        # Test assertions
        self.assertIsNone(resource)

    def test_resource_show_with_not_admin_and_resource_in_project(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_1', 'is_admin': False}

        # Action
        resource = apis.show_resource(ctx, 'instance_2')
        resource = json.loads(resource)

        # Test assertions
        self.assertIsNotNone(resource)
        self._check_resource_properties(resource, 'instance_2',
                                        NOVA_INSTANCE_DATASOURCE,
                                        project_id='project_1')

    def test_resource_show_with_not_admin_and_resource_in_other_project(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': False}

        # Action
        resource = apis.show_resource(ctx, 'instance_2')

        # Test assertions
        self.assertIsNone(resource)

    def test_resource_show_with_admin_and_resource_in_project(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_1', 'is_admin': True}

        # Action
        resource = apis.show_resource(ctx, 'instance_2')
        resource = json.loads(resource)

        # Test assertions
        self.assertIsNotNone(resource)
        self._check_resource_properties(resource, 'instance_2',
                                        NOVA_INSTANCE_DATASOURCE,
                                        project_id='project_1')

    def test_resource_show_with_admin_and_resource_in_other_project(self):
        # Setup
        graph = self._create_graph()
        apis = ResourceApis(graph, self.api_lock)
        ctx = {'project_id': 'project_2', 'is_admin': True}

        # Action
        resource = apis.show_resource(ctx, 'instance_2')
        resource = json.loads(resource)

        # Test assertions
        self.assertIsNotNone(resource)
        self._check_resource_properties(resource, 'instance_2',
                                        NOVA_INSTANCE_DATASOURCE,
                                        project_id='project_1')

    def _check_projects_entities(self,
                                 alarms,
                                 project_id,
                                 check_alarm_category):
        for alarm in alarms:
            tmp_project_id = alarm.get(VProps.PROJECT_ID, None)
            condition = True
            if check_alarm_category:
                condition = \
                    alarm[VProps.VITRAGE_CATEGORY] == EntityCategory.ALARM
            if project_id:
                condition = condition and \
                    (not tmp_project_id or
                     (tmp_project_id and tmp_project_id == project_id))
            self.assertTrue(condition)

    def _check_resource_properties(self, resource, vitrage_id,
                                   resource_type, project_id=None):
        self.assertEqual(resource[VProps.VITRAGE_ID], vitrage_id)
        self.assertEqual(resource[VProps.ID], vitrage_id)
        self.assertEqual(resource[VProps.VITRAGE_CATEGORY],
                         EntityCategory.RESOURCE)
        self.assertEqual(resource[VProps.VITRAGE_TYPE], resource_type)
        self.assertEqual(resource[VProps.STATE], 'active')
        self.assertFalse(resource[VProps.VITRAGE_IS_DELETED])
        self.assertFalse(resource[VProps.VITRAGE_IS_PLACEHOLDER])
        if project_id:
            self.assertEqual(resource[VProps.PROJECT_ID], project_id)

    def _create_graph(self):
        graph = NXGraph('Multi tenancy graph')
        self._add_alarm_persistency_subscription(graph)

        # create vertices
        cluster_vertex = create_cluster_placeholder_vertex()
        zone_vertex = self._create_resource('zone_1',
                                            NOVA_ZONE_DATASOURCE)
        host_vertex = self._create_resource('host_1',
                                            NOVA_HOST_DATASOURCE)
        instance_1_vertex = self._create_resource('instance_1',
                                                  NOVA_INSTANCE_DATASOURCE,
                                                  project_id='project_1')
        instance_2_vertex = self._create_resource('instance_2',
                                                  NOVA_INSTANCE_DATASOURCE,
                                                  project_id='project_1')
        instance_3_vertex = self._create_resource('instance_3',
                                                  NOVA_INSTANCE_DATASOURCE,
                                                  project_id='project_2')
        instance_4_vertex = self._create_resource('instance_4',
                                                  NOVA_INSTANCE_DATASOURCE,
                                                  project_id='project_2')
        alarm_on_host_vertex = self._create_alarm(
            'alarm_on_host',
            'alarm_on_host',
            metadata={VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE,
                      VProps.NAME: 'host_1',
                      VProps.RESOURCE_ID: 'host_1',
                      VProps.VITRAGE_OPERATIONAL_SEVERITY:
                          OperationalAlarmSeverity.SEVERE,
                      VProps.VITRAGE_AGGREGATED_SEVERITY:
                          OperationalAlarmSeverity.SEVERE})
        alarm_on_instance_1_vertex = self._create_alarm(
            'alarm_on_instance_1',
            'deduced_alarm',
            project_id='project_1',
            vitrage_resource_project_id='project_1',
            metadata={VProps.VITRAGE_TYPE: NOVA_INSTANCE_DATASOURCE,
                      VProps.NAME: 'instance_1',
                      VProps.RESOURCE_ID: 'sdg7849ythksjdg',
                      VProps.VITRAGE_OPERATIONAL_SEVERITY:
                          OperationalAlarmSeverity.SEVERE,
                      VProps.VITRAGE_AGGREGATED_SEVERITY:
                          OperationalAlarmSeverity.SEVERE})
        alarm_on_instance_2_vertex = self._create_alarm(
            'alarm_on_instance_2',
            'deduced_alarm',
            vitrage_resource_project_id='project_1',
            metadata={VProps.VITRAGE_TYPE: NOVA_INSTANCE_DATASOURCE,
                      VProps.NAME: 'instance_2',
                      VProps.RESOURCE_ID: 'nbfhsdugf',
                      VProps.VITRAGE_OPERATIONAL_SEVERITY:
                          OperationalAlarmSeverity.WARNING,
                      VProps.VITRAGE_AGGREGATED_SEVERITY:
                          OperationalAlarmSeverity.WARNING})
        alarm_on_instance_3_vertex = self._create_alarm(
            'alarm_on_instance_3',
            'deduced_alarm',
            project_id='project_2',
            vitrage_resource_project_id='project_2',
            metadata={VProps.VITRAGE_TYPE: NOVA_INSTANCE_DATASOURCE,
                      VProps.NAME: 'instance_3',
                      VProps.RESOURCE_ID: 'nbffhsdasdugf',
                      VProps.VITRAGE_OPERATIONAL_SEVERITY:
                          OperationalAlarmSeverity.CRITICAL,
                      VProps.VITRAGE_AGGREGATED_SEVERITY:
                          OperationalAlarmSeverity.CRITICAL})
        alarm_on_instance_4_vertex = self._create_alarm(
            'alarm_on_instance_4',
            'deduced_alarm',
            vitrage_resource_project_id='project_2',
            metadata={VProps.VITRAGE_TYPE: NOVA_INSTANCE_DATASOURCE,
                      VProps.NAME: 'instance_4',
                      VProps.RESOURCE_ID: 'ngsuy76hgd87f',
                      VProps.VITRAGE_OPERATIONAL_SEVERITY:
                          OperationalAlarmSeverity.WARNING,
                      VProps.VITRAGE_AGGREGATED_SEVERITY:
                          OperationalAlarmSeverity.WARNING})

        # create links
        edges = list()
        edges.append(graph_utils.create_edge(
            cluster_vertex.vertex_id,
            zone_vertex.vertex_id,
            EdgeLabel.CONTAINS,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            zone_vertex.vertex_id,
            host_vertex.vertex_id,
            EdgeLabel.CONTAINS,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            host_vertex.vertex_id,
            instance_1_vertex.vertex_id,
            EdgeLabel.CONTAINS,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            host_vertex.vertex_id,
            instance_2_vertex.vertex_id,
            EdgeLabel.CONTAINS,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            host_vertex.vertex_id,
            instance_3_vertex.vertex_id,
            EdgeLabel.CONTAINS,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            host_vertex.vertex_id,
            instance_4_vertex.vertex_id,
            EdgeLabel.CONTAINS,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            alarm_on_host_vertex.vertex_id,
            host_vertex.vertex_id,
            EdgeLabel.ON,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            alarm_on_instance_1_vertex.vertex_id,
            instance_1_vertex.vertex_id,
            EdgeLabel.ON,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            alarm_on_instance_2_vertex.vertex_id,
            instance_2_vertex.vertex_id,
            EdgeLabel.ON,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            alarm_on_instance_3_vertex.vertex_id,
            instance_3_vertex.vertex_id,
            EdgeLabel.ON,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            alarm_on_instance_4_vertex.vertex_id,
            instance_4_vertex.vertex_id,
            EdgeLabel.ON,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            alarm_on_host_vertex.vertex_id,
            alarm_on_instance_1_vertex.vertex_id,
            EdgeLabel.CAUSES,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            alarm_on_host_vertex.vertex_id,
            alarm_on_instance_2_vertex.vertex_id,
            EdgeLabel.CAUSES,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            alarm_on_host_vertex.vertex_id,
            alarm_on_instance_3_vertex.vertex_id,
            EdgeLabel.CAUSES,
            update_timestamp=str(utcnow())))
        edges.append(graph_utils.create_edge(
            alarm_on_host_vertex.vertex_id,
            alarm_on_instance_4_vertex.vertex_id,
            EdgeLabel.CAUSES,
            update_timestamp=str(utcnow())))

        # add vertices to graph
        graph.add_vertex(cluster_vertex)
        graph.add_vertex(zone_vertex)
        graph.add_vertex(host_vertex)
        graph.add_vertex(instance_1_vertex)
        graph.add_vertex(instance_2_vertex)
        graph.add_vertex(instance_3_vertex)
        graph.add_vertex(instance_4_vertex)
        graph.add_vertex(alarm_on_host_vertex)
        graph.add_vertex(alarm_on_instance_1_vertex)
        graph.add_vertex(alarm_on_instance_2_vertex)
        graph.add_vertex(alarm_on_instance_3_vertex)
        graph.add_vertex(alarm_on_instance_4_vertex)

        # add links to graph
        for edge in edges:
            graph.add_edge(edge)

        return graph

    def _add_alarm_persistency_subscription(self, graph):

        self._db.alarms.delete()
        self._db.changes.delete()
        self._db.edges.delete()
        persistor_endpoint = VitragePersistorEndpoint(self._db)

        def callback(before, curr, is_vertex, graph):
            notification_types = PersistNotifier._get_notification_type(
                before, curr, is_vertex)
            if not is_vertex:
                curr = edge_copy(
                    curr.source_id, curr.target_id, curr.label,
                    curr.properties)
                curr.properties[EdgeProperties.SOURCE_ID] = curr.source_id
                curr.properties[EdgeProperties.TARGET_ID] = curr.target_id

            for notification_type in notification_types:
                persistor_endpoint.process_event(notification_type,
                                                 curr.properties)

        graph.subscribe(callback)



# Copyright 2017 - ZTE Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from vitrage.common.constants import TemplateTypes
from vitrage.common.exception import VitrageError
from vitrage.evaluator.equivalence_repository import EquivalenceRepository
from vitrage.tests import base
from vitrage.tests.functional.test_configuration import TestConfiguration
from vitrage.tests.mocks import utils


class TestEquivalenceRepository(base.BaseTest, TestConfiguration):

    # noinspection PyPep8Naming
    def setUp(self):
        super(TestEquivalenceRepository, self).setUp()
        self.add_db()
        self.equivalence_repository = EquivalenceRepository()

    def test_duplicate_entities_in_equivalence(self):
        base_dir = utils.get_resources_dir() + '/templates/equivalences_dup'
        for directory in os.listdir(base_dir):
            self.add_templates(os.path.join(base_dir, directory),
                               TemplateTypes.EQUIVALENCE)
            self.assertRaises(
                VitrageError,
                self.equivalence_repository.load, self._db)



# Copyright 2016 - Alcatel-Lucent
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime

from oslo_config import cfg
from oslo_log import log as logging
from testtools import matchers

from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EdgeLabel
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import UpdateMethod
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE
from vitrage.datasources.nova.host.transformer import HostTransformer
from vitrage.datasources.nova.zone import NOVA_ZONE_DATASOURCE
from vitrage.datasources.nova.zone.transformer import ZoneTransformer
from vitrage.datasources import OPENSTACK_CLUSTER
from vitrage.datasources import transformer_base as tbase
from vitrage.datasources.transformer_base import CLUSTER_ID
from vitrage.tests import base
from vitrage.tests.mocks import mock_driver as mock_sync

LOG = logging.getLogger(__name__)


# noinspection PyProtectedMember
class NovaZoneTransformerTest(base.BaseTest):

    OPTS = [
        cfg.StrOpt(DSOpts.UPDATE_METHOD,
                   default=UpdateMethod.PUSH),
    ]

    # noinspection PyAttributeOutsideInit,PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(NovaZoneTransformerTest, cls).setUpClass()
        cls.transformers = {}
        cls.conf = cfg.ConfigOpts()
        cls.conf.register_opts(cls.OPTS, group=NOVA_ZONE_DATASOURCE)
        cls.transformers[NOVA_HOST_DATASOURCE] = \
            HostTransformer(cls.transformers)
        cls.transformers[NOVA_ZONE_DATASOURCE] = \
            ZoneTransformer(cls.transformers)

    def test_create_placeholder_vertex(self):

        LOG.debug('Zone transformer test: create placeholder vertex')

        # Test setup
        zone_name = 'zone123'
        timestamp = datetime.datetime.utcnow()
        zone_transformer = self.transformers[NOVA_ZONE_DATASOURCE]

        # Test action
        properties = {
            VProps.ID: zone_name,
            VProps.VITRAGE_TYPE: NOVA_ZONE_DATASOURCE,
            VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE,
            VProps.VITRAGE_SAMPLE_TIMESTAMP: timestamp
        }
        placeholder = \
            zone_transformer.create_neighbor_placeholder_vertex(**properties)

        # Test assertions
        observed_uuid = placeholder.vertex_id
        expected_key = tbase.build_key(zone_transformer._key_values(
            NOVA_ZONE_DATASOURCE,
            zone_name))
        expected_uuid = \
            zone_transformer.uuid_from_deprecated_vitrage_id(expected_key)
        self.assertEqual(expected_uuid, observed_uuid)

        observed_time = placeholder.get(VProps.VITRAGE_SAMPLE_TIMESTAMP)
        self.assertEqual(timestamp, observed_time)

        observed_subtype = placeholder.get(VProps.VITRAGE_TYPE)
        self.assertEqual(NOVA_ZONE_DATASOURCE, observed_subtype)

        observed_entity_id = placeholder.get(VProps.ID)
        self.assertEqual(zone_name, observed_entity_id)

        observed_vitrage_category = placeholder.get(VProps.VITRAGE_CATEGORY)
        self.assertEqual(EntityCategory.RESOURCE, observed_vitrage_category)

        vitrage_is_placeholder = placeholder.get(VProps.VITRAGE_IS_PLACEHOLDER)
        self.assertTrue(vitrage_is_placeholder)

    def test_key_values(self):
        LOG.debug('Zone transformer test: get key values')

        # Test setup
        zone_name = 'zone123'
        zone_transformer = self.transformers[NOVA_ZONE_DATASOURCE]

        # Test action
        observed_key_fields = zone_transformer._key_values(
            NOVA_ZONE_DATASOURCE,
            zone_name)

        # Test assertions
        self.assertEqual(EntityCategory.RESOURCE, observed_key_fields[0])
        self.assertEqual(
            NOVA_ZONE_DATASOURCE,
            observed_key_fields[1]
        )
        self.assertEqual(zone_name, observed_key_fields[2])

    def test_create_entity_key(self):
        pass

    def test_snapshot_transform(self):
        LOG.debug('Nova zone transformer test: transform entity event')

        # Test setup
        spec_list = mock_sync.simple_zone_generators(zone_num=1,
                                                     host_num=1,
                                                     snapshot_events=5)
        zone_events = mock_sync.generate_random_events_list(spec_list)

        for event in zone_events:
            # Test action
            wrapper = self.transformers[NOVA_ZONE_DATASOURCE].transform(event)

            # Test assertions
            vertex = wrapper.vertex
            self._validate_vertex_props(vertex, event)

            neighbors = wrapper.neighbors
            self.assertThat(neighbors, matchers.HasLength(2))
            self._validate_neighbors(neighbors, vertex.vertex_id, event)

    def _validate_neighbors(self, neighbors, zone_vertex_id, event):

        cluster_neighbors_counter = 0

        for neighbor in neighbors:
            vertex_type = neighbor.vertex.get(VProps.VITRAGE_TYPE)

            if OPENSTACK_CLUSTER == vertex_type:
                cluster_neighbors_counter += 1
                self._validate_cluster_neighbor(neighbor, zone_vertex_id)
            else:
                hosts = tbase.extract_field_value(event, 'hosts')
                self._validate_host_neighbor(neighbor,
                                             zone_vertex_id,
                                             hosts,
                                             event[DSProps.DATASOURCE_ACTION])

        self.assertEqual(1,
                         cluster_neighbors_counter,
                         'Zone can belongs to only one Cluster')

    def _validate_host_neighbor(self,
                                host_neighbor,
                                zone_vertex_id,
                                hosts,
                                datasource_action):

        host_vertex = host_neighbor.vertex
        host_vertex_id = host_vertex.get(VProps.ID)

        host_dic = hosts[host_vertex_id]
        self.assertIsNotNone(hosts[host_vertex_id])

        host_available = tbase.extract_field_value(
            host_dic,
            'nova-compute', 'available'
        )
        host_active = tbase.extract_field_value(
            host_dic,
            'nova-compute', 'active'
        )

        if host_available and host_active:
            expected_host_state = ZoneTransformer.STATE_AVAILABLE
        else:
            expected_host_state = ZoneTransformer.STATE_UNAVAILABLE
        self.assertEqual(
            expected_host_state,
            host_vertex.get(VProps.STATE)
        )

        vitrage_is_placeholder = host_vertex[VProps.VITRAGE_IS_PLACEHOLDER]
        self.assertFalse(vitrage_is_placeholder)

        vitrage_is_deleted = host_vertex[VProps.VITRAGE_IS_DELETED]
        self.assertFalse(vitrage_is_deleted)

        # Validate neighbor edge
        edge = host_neighbor.edge
        self.assertEqual(edge.target_id, host_neighbor.vertex.vertex_id)
        self.assertEqual(edge.source_id, zone_vertex_id)
        self.assertEqual(edge.label, EdgeLabel.CONTAINS)

    def _validate_cluster_neighbor(self, cluster_neighbor, zone_vertex_id):

        self.assertEqual(cluster_neighbor.vertex[VProps.VITRAGE_ID],
                         cluster_neighbor.vertex.vertex_id)
        self.assertFalse(cluster_neighbor.vertex[VProps.VITRAGE_IS_DELETED])
        self.assertEqual(EntityCategory.RESOURCE,
                         cluster_neighbor.vertex[VProps.VITRAGE_CATEGORY])
        self.assertEqual(CLUSTER_ID,
                         cluster_neighbor.vertex[VProps.ID])
        self.assertFalse(cluster_neighbor.vertex[
            VProps.VITRAGE_IS_PLACEHOLDER])
        self.assertEqual(OPENSTACK_CLUSTER,
                         cluster_neighbor.vertex[VProps.VITRAGE_TYPE])

        # Validate neighbor edge
        edge = cluster_neighbor.edge
        self.assertEqual(edge.source_id, cluster_neighbor.vertex.vertex_id)
        self.assertEqual(edge.target_id, zone_vertex_id)
        self.assertEqual(edge.label, EdgeLabel.CONTAINS)

    def _validate_vertex_props(self, vertex, event):

        zone_transform = self.transformers[NOVA_ZONE_DATASOURCE]

        extract_value = tbase.extract_field_value

        expected_id = extract_value(event, 'zoneName')
        observed_id = vertex[VProps.ID]
        self.assertEqual(expected_id, observed_id)

        self.assertEqual(EntityCategory.RESOURCE,
                         vertex[VProps.VITRAGE_CATEGORY])

        self.assertEqual(NOVA_ZONE_DATASOURCE,
                         vertex[VProps.VITRAGE_TYPE])

        expected_timestamp = event[DSProps.SAMPLE_DATE]
        observed_timestamp = vertex[VProps.VITRAGE_SAMPLE_TIMESTAMP]
        self.assertEqual(expected_timestamp, observed_timestamp)

        expected_name = extract_value(event, 'zoneName')
        observed_name = vertex[VProps.NAME]
        self.assertEqual(expected_name, observed_name)

        is_zone_available = extract_value(event, 'zoneState', 'available')

        if is_zone_available:
            expected_state = zone_transform.STATE_AVAILABLE
        else:
            expected_state = zone_transform.STATE_UNAVAILABLE

        observed_state = vertex[VProps.STATE]
        self.assertEqual(expected_state, observed_state)

        vitrage_is_placeholder = vertex[VProps.VITRAGE_IS_PLACEHOLDER]
        self.assertFalse(vitrage_is_placeholder)

        vitrage_is_deleted = vertex[VProps.VITRAGE_IS_DELETED]
        self.assertFalse(vitrage_is_deleted)



# Copyright 2018 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.common.constants import EdgeProperties
from vitrage.common.constants import VertexProperties
from vitrage.graph.driver.networkx_graph import NXGraph

from vitrage.entity_graph import graph_persistency
from vitrage.tests.functional.base import TestFunctionalBase
from vitrage.tests.functional.test_configuration import TestConfiguration
from vitrage.tests.mocks.graph_generator import GraphGenerator


class TestGraphPersistor(TestFunctionalBase, TestConfiguration):

    def setUp(self):
        super(TestGraphPersistor, self).setUp()
        self.conf.register_opts(self.PROCESSOR_OPTS, group='entity_graph')
        self.conf.register_opts(self.DATASOURCES_OPTS, group='datasources')
        self.add_db()
        self.load_datasources()

    def test_graph_store_and_query_recent_snapshot(self):
        g = GraphGenerator().create_graph()
        graph_persistor = graph_persistency.GraphPersistency(self._db, g)
        graph_persistor.store_graph()
        recovered_data = graph_persistor.query_recent_snapshot()
        recovered_graph = self.load_snapshot(recovered_data)
        self.assert_graph_equal(g, recovered_graph)

    def test_event_store_and_replay_events(self):
        g = GraphGenerator().create_graph()
        vertices = g.get_vertices()
        graph_persistor = graph_persistency.GraphPersistency(self._db, g)
        self.fail_msg = None
        self.event_id = 1

        def callback(pre_item,
                     current_item,
                     is_vertex,
                     graph):
            try:
                graph_persistor.persist_event(
                    pre_item, current_item, is_vertex, graph, self.event_id)
            except Exception as e:
                self.fail_msg = 'persist_event failed with exception %s' % e
            self.event_id = self.event_id + 1

        # Subscribe graph changes to callback, so events are written to db
        # after each update_vertex and update_edge callback will be called
        g.subscribe(callback)
        vertices[0][VertexProperties.VITRAGE_IS_DELETED] = True
        g.update_vertex(vertices[0])
        vertices[1][VertexProperties.VITRAGE_IS_DELETED] = True
        g.update_vertex(vertices[1])
        edge = g.get_edges(vertices[0].vertex_id).pop()
        edge[EdgeProperties.VITRAGE_IS_DELETED] = True
        g.update_edge(edge)
        graph_persistor.flush_events()

        # Store graph:
        graph_persistor.store_graph()

        # Create more events:
        vertices[2][VertexProperties.VITRAGE_IS_DELETED] = True
        g.update_vertex(vertices[2])
        vertices[3][VertexProperties.VITRAGE_IS_DELETED] = True
        g.update_vertex(vertices[3])
        edge = g.get_edges(vertices[2].vertex_id).pop()
        edge[EdgeProperties.RELATIONSHIP_TYPE] = 'kuku'
        g.update_edge(edge)
        graph_persistor.flush_events()

        self.assertIsNone(self.fail_msg, 'callback failed')

        # Reload snapshot
        recovered_data = graph_persistor.query_recent_snapshot()
        recovered_graph = self.load_snapshot(recovered_data)

        # Replay events:
        self.assertEqual(3, recovered_data.event_id, 'graph snapshot event_id')
        graph_persistor.replay_events(recovered_graph, recovered_data.event_id)

        self.assert_graph_equal(g, recovered_graph)

    @staticmethod
    def load_snapshot(data):
        return NXGraph.read_gpickle(data.graph_snapshot) if data else None



# Copyright 2018 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import ast
from collections import namedtuple
import datetime
from oslo_log import log
from oslo_utils import uuidutils
from osprofiler import profiler
import re
from urllib.parse import urlparse
from vitrage.common.constants import TenantProps
from vitrage.common.constants import VertexProperties as Vprops
from vitrage.notifier.plugins.webhook.utils import db_row_to_dict
from vitrage.storage.sqlalchemy.models import Webhooks

LOG = log.getLogger(__name__)

Result = namedtuple("Result", ["is_valid", "message"])


@profiler.trace_cls("webhook apis",
                    info={}, hide_args=False, trace_private=False)
class WebhookApis(object):
    DELETED_ROWS_SUCCESS = 1

    def __init__(self, db):
        self.db_conn = db

    def delete_webhook(self, ctx, id):

        LOG.info("Delete webhook with id: %s", id)

        deleted_rows_count = self.db_conn.webhooks.delete(id)

        if deleted_rows_count == self.DELETED_ROWS_SUCCESS:
            return {'SUCCESS': 'Webhook %s deleted' % id}
        else:
            return None

    def get_all_webhooks(self, ctx, all_tenants):
        LOG.info("List all webhooks")
        if all_tenants and ctx.get(TenantProps.IS_ADMIN, False):
            res = self.db_conn.webhooks.query()
        else:
            res = self.db_conn.webhooks.query(project_id=ctx.get(
                TenantProps.PROJECT_ID, ""))
        LOG.info(res)
        webhooks = [db_row_to_dict(webhook) for webhook in res]

        return webhooks

    def add_webhook(self, ctx, url, headers=None, regex_filter=None):
        res = self._check_valid_webhook(url, headers, regex_filter)
        if not res.is_valid:
            LOG.exception("Failed to create webhook: %s" % res.message)
            return res.message
        try:
            db_row = self._webhook_to_db_row(url, headers, regex_filter, ctx)
            self.db_conn.webhooks.create(db_row)
            return db_row_to_dict(db_row)
        except Exception as e:
            LOG.exception("Failed to add webhook to DB")
            return {"ERROR": str(e)}

    def get_webhook(self, ctx, id):
        try:
            webhooks = self.db_conn.webhooks.query(id=id)
            # Check that webhook belongs to current tenant or current tenant
            #  is admin
            if len(webhooks) == 0:
                LOG.warning("Webhook not found - %s" % id)
                return None
            if ctx.get(TenantProps.PROJECT_ID, "") == \
                    webhooks[0][Vprops.PROJECT_ID] or ctx.get(
                    TenantProps.IS_ADMIN, False):
                return webhooks[0]
            else:
                LOG.warning('Webhook show - Authorization failed (%s)',
                            id)
                return None
        except Exception as e:
            LOG.exception("Failed to get webhook")
            return {"ERROR": str(e)}

    def _webhook_to_db_row(self, url, headers, regex_filter, ctx):
        if not regex_filter:
            regex_filter = ""
        if not headers:
            headers = ""
        uuid = uuidutils.generate_uuid()
        project_id = ctx.get(TenantProps.PROJECT_ID, "")
        is_admin = ctx.get(TenantProps.IS_ADMIN, False)
        created_at = str(datetime.datetime.now())
        db_row = Webhooks(id=uuid,
                          project_id=project_id,
                          is_admin_webhook=is_admin,
                          created_at=created_at,
                          url=url,
                          headers=headers,
                          regex_filter=regex_filter)
        return db_row

    def _check_valid_webhook(self, url, headers, regex_filter):
        if not self._validate_url(url):
            return Result(False, {"ERROR": "Invalid URL"})
        elif not self._validate_headers(headers):
            return Result(False, {"ERROR": "Headers in invalid format"})
        elif not self._validate_regex(regex_filter):
            return Result(False, {"ERROR": "Invalid RegEx"})
        return Result(True, "")

    def _validate_url(self, url):
        try:
            result = urlparse(url)
            if not result.scheme or not result.netloc:
                return False
        except Exception:
            return False
        return True

    def _validate_regex(self, regex_filter):
        if regex_filter:
            try:
                filter_dict = ast.literal_eval(regex_filter)
                if not isinstance(filter_dict, dict):
                    return False
                for filter in filter_dict.values():
                    re.compile(filter)
            except Exception:
                return False
        return True

    def _validate_headers(self, headers):
        if headers:
            try:
                return isinstance(ast.literal_eval(headers), dict)
            except Exception:
                return False
        return True



# Copyright 2017 - Nokia Corporation
# Copyright 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# noinspection PyPackageRequirements
from datetime import datetime
from unittest import mock
import uuid


from keystonemiddleware import fixture as ksm_fixture
# noinspection PyPackageRequirements
from vitrage.tests.functional.api.v1 import FunctionalTest

EVENT_DETAILS = {
    'hostname': 'host123',
    'source': 'sample_monitor',
    'cause': 'another alarm',
    'severity': 'critical',
    'status': 'down',
    'monitor_id': 'sample monitor',
    'monitor_event_id': '456',
}

VALID_TOKEN = uuid.uuid4().hex

HEADERS = {
    'X-Auth-Token': VALID_TOKEN,
    'X-Project-Id': 'admin',
    'X-Roles': 'admin'
}


class AuthTest(FunctionalTest):

    def __init__(self, *args, **kwds):
        super(AuthTest, self).__init__(*args, **kwds)
        self.auth = 'keystone'

    def setUp(self):
        super(AuthTest, self).setUp()
        self.auth_token_fixture = self.useFixture(
            ksm_fixture.AuthTokenFixture())
        self.auth_token_fixture.add_token_data(
            token_id=VALID_TOKEN,
            project_id='admin',
            role_list=['admin'],
            user_name='user_name',
            user_id='user_id',
            is_v2=True
        )

    def test_in_keystone_mode_not_authenticated(self):
        resp = self.post_json('/topology/', params=None, expect_errors=True)
        self.assertEqual('401 Unauthorized', resp.status)

    def test_in_keystone_mode_auth_success(self):
        with mock.patch('pecan.request') as request:
            resp = self.post_json('/event/',
                                  params={
                                      'time': datetime.now().isoformat(),
                                      'type': 'compute.host.down',
                                      'details': EVENT_DETAILS
                                  },
                                  headers=HEADERS)

            self.assertEqual(1, request.client.call.call_count)
            self.assertEqual('200 OK', resp.status)



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy
import sys

import cotyledon
from futurist import periodics
from futurist import ThreadPoolExecutor

from oslo_config import cfg
from oslo_log import log
import oslo_messaging
from tools.load_generator.notification_info import COMPUTE_INSTANCE_CREATE_END
from tools.load_generator.notification_info import PORT_CREATE_END
from tools.load_generator.notification_info import VOLUME_ATTACH_END
from tools.load_generator.notification_info import VOLUME_CREATE_END

from vitrage.common import config
from vitrage.messaging import get_transport


CONF = cfg.CONF
LOG = log.getLogger(__name__)

EXISTING_COMPUTES_NUM = 64
VMS_PER_COMPUTE = 2
NET_ID = '59fec1a4-7ab2-4bc6-8792-0ddf54b15dfe'
RUN_EVERY_X_SECONDS = 600

"""
Stress Notifications Tool:

Following service runs a timed action every X seconds.
Action will send mock bus notifications, as configured in the constants above.
Sends mock notifications for:
    VM create
    Port create
    volume create
    volume attach

1. To use this, place computes.yaml at /etc/vitrage/static_datasources/
   and restart vitrage-graph.
2. EXISTING_COMPUTES_NUM should match the computes defined in computes.yaml
3. Configure NET_ID to an existing network (this tool doesnt create networks)
4. Run 'python load_generator.py'

Number of vms = VMS_PER_COMPUTE * EXISTING_COMPUTES_NUM
Number of ports = VMS_PER_COMPUTE * EXISTING_COMPUTES_NUM
Number of volumes = VMS_PER_COMPUTE * EXISTING_COMPUTES_NUM

Notifications are sent repeatedly every RUN_EVERY_X_SECONDS, this is
to avoid Vitrage consistency deleting the created resources.

* Folder /templates also includes templates to create load on the evaluator

"""


class StressNotificationsService(cotyledon.Service):
    def __init__(self, worker_id):
        super(StressNotificationsService, self).__init__(worker_id)
        self.oslo_notifier = None
        topics = CONF.datasources.notification_topics
        self.oslo_notifier = oslo_messaging.Notifier(
            get_transport(),
            driver='messagingv2',
            publisher_id='vitrage.stress',
            topics=topics)
        self.periodic = periodics.PeriodicWorker.create(
            [], executor_factory=lambda: ThreadPoolExecutor(max_workers=10))

    def run(self):
        LOG.info("StressNotificationsService - Started!")
        self.periodic.add(self.stress_notifications)
        self.periodic.start()

    def terminate(self):
        self.periodic.stop()
        LOG.info("StressNotificationsService - Stopped!")

    @periodics.periodic(spacing=RUN_EVERY_X_SECONDS)
    def stress_notifications(self):
        notifications = []
        for i in range(EXISTING_COMPUTES_NUM * VMS_PER_COMPUTE):
            vm = create_vm(i, i % EXISTING_COMPUTES_NUM)
            port = create_port(i, vm[0][1]['instance_id'], vm[0][1]['host'],
                               NET_ID)
            storage = create_storage(i, vm[0][1]['instance_id'])
            notifications.extend(vm)
            notifications.extend(port)
            notifications.extend(storage)

        LOG.info("Notifications Created - " + str(len(notifications)))
        LOG.info("Sending...")
        for n in notifications:
            self._send(*n)
        LOG.info("Sent!")

    def _send(self, notification_type, payload):
        try:
            self.oslo_notifier.info(
                {},
                notification_type,
                payload)
        except Exception:
            LOG.exception('Cannot notify - %s', notification_type)


def create_port(port_num, instance_id, host_id, net_id):
    payload = copy.deepcopy(PORT_CREATE_END)
    payload['port']['id'] = 'StressPort-' + str(port_num)
    payload['port']['device_id'] = instance_id
    payload['port']['binding:host_id'] = host_id
    payload['port']['network_id'] = net_id
    return [('port.create.end', payload)]


def create_storage(volume_num, instance_id):
    payload_1 = copy.deepcopy(VOLUME_CREATE_END)
    payload_1['volume_id'] = 'StressVolume-' + str(volume_num)

    payload_2 = copy.deepcopy(VOLUME_ATTACH_END)
    payload_2['volume_id'] = payload_1['volume_id']
    payload_2['volume_attachment'][0]['volume']['id'] = payload_1['volume_id']
    payload_2['volume_attachment'][0]['instance_uuid'] = instance_id

    return [('volume.create.end', payload_1),
            ('volume.attach.end', payload_2)]


def create_vm(instance_num, compute_num):
    payload = copy.deepcopy(COMPUTE_INSTANCE_CREATE_END)
    payload['instance_id'] = 'StressVM-' + str(instance_num)
    payload['node'] = payload['host'] = "compute-0-" + str(compute_num)
    return [('compute.instance.create.end', payload)]


def main():
    config.parse_config(sys.argv)
    sm = cotyledon.ServiceManager()
    sm.add(StressNotificationsService)
    sm.run()


if __name__ == "__main__":
    sys.exit(main())



# Copyright 2016 - Alcatel-Lucent
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg

from vitrage.evaluator.template_schemas import init_template_schemas

# Register options for the service
OPTS = [
    cfg.IntOpt('workers',
               default=1,
               min=1,
               max=32,
               help='Number of workers for template evaluator.'
               ),
]

init_template_schemas()



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime
from oslo_log import log

LOG = log.getLogger(__name__)


class AlarmDataAccumulator(object):

    def __init__(self, accumulated_data):
        self.active_start_times = {}
        self.alarms_activity = accumulated_data.activity
        # TODO(annarez): exclude intersections between deduced and it's cause
        self.alarms_intersects = accumulated_data.intersection

    def append_active(self, alarm_id, timestamp):

        if alarm_id in self.active_start_times:
            LOG.debug("Active alarm %s was started twice. Second time at %s",
                      alarm_id, timestamp)
            return

        self.active_start_times[alarm_id] = timestamp

    def append_inactive(self, alarm_id, end_time):

        if alarm_id not in self.active_start_times:
            LOG.debug("Alarm {} at {} was deactivated without being active".
                      format(alarm_id, str(end_time)))
            return

        alarm_duration = end_time - self.active_start_times[alarm_id]
        self.alarms_activity[alarm_id] = alarm_duration + \
            self.alarms_activity.get(alarm_id, datetime.timedelta(0))

        start_time = self.active_start_times[alarm_id]

        del self.active_start_times[alarm_id]
        self.append_intersect(alarm_id, start_time, end_time)

    def append_intersect(self, alarm_id, start_time, end_time):

        for active_alarm in self.active_start_times.items():
            key = frozenset([alarm_id, active_alarm[0]])
            active_alarm_service = active_alarm[1]
            self.alarms_intersects[key] = \
                self.alarms_intersects.get(key, datetime.timedelta(0)) + \
                self.calc_intersect(start_time, end_time, active_alarm_service)

    def calc_intersect(self, inactive_start, inactive_end, active_start):
        return inactive_end - max(inactive_start, active_start)

    def flush_accumulations(self):
        """flush all active alarms

        empty the data from active_start_times and re-enter all the
        currently-active alarms, as if they started now
        """

        now = datetime.datetime.now()
        active_alarms = list(self.active_start_times)

        for active_alarm in active_alarms:
            self.append_inactive(active_alarm, now)

        for flushed_active_alarm in active_alarms:
            self.append_active(flushed_active_alarm, now)



# Copyright 2017 - Nokia Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan

from oslo_log import log
from osprofiler import profiler
from pecan.core import abort

from datetime import datetime
from vitrage.api.controllers.rest import RootRestController
from vitrage.api.policy import enforce
from vitrage.datasources.prometheus.driver import PROMETHEUS_EVENT_TYPE


LOG = log.getLogger(__name__)


# noinspection PyBroadException
@profiler.trace_cls("event controller",
                    info={}, hide_args=False, trace_private=False)
class EventController(RootRestController):

    @pecan.expose('json')
    def post(self, **kwargs):
        LOG.info('Post event called with args: %s', kwargs)

        enforce("event post", pecan.request.headers,
                pecan.request.enforcer, {})

        prom_event_type = None
        user_agent = pecan.request.headers.get('User-Agent')
        if user_agent and user_agent.startswith("Alertmanager"):
            prom_event_type = PROMETHEUS_EVENT_TYPE

        event_time = kwargs.get('time', datetime.utcnow())
        event_type = kwargs.get('type', prom_event_type)
        details = kwargs.get('details', kwargs)

        self.post_event(event_time, event_type, details)

    @staticmethod
    def post_event(event_time, event_type, details):
        try:
            pecan.request.client.call(pecan.request.context,
                                      'post',
                                      event_time=event_time,
                                      event_type=event_type,
                                      details=details)
        except Exception:
            LOG.exception('Failed to post an event')
            abort(404, 'Failed to post event')



# Copyright 2019 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log

from vitrage.evaluator.template_functions import function_resolver
from vitrage.evaluator.template_functions import GET_PARAM
from vitrage.evaluator.template_validation.base import get_custom_fault_result
from vitrage.evaluator.template_validation.base import ValidationError
from vitrage.evaluator.template_validation.content.base import \
    get_content_correct_result
from vitrage.evaluator.template_validation.content.base import \
    get_template_schema


LOG = log.getLogger(__name__)


def resolve_parameters(template_def, params=None):
    result, template_schema = get_template_schema(template_def)
    if not result.is_valid_config:
        return result

    get_param = template_schema.functions.get(GET_PARAM)

    try:
        function_resolver.resolve_function(
            func_info=function_resolver.FuncInfo(
                name=GET_PARAM, func=get_param, error_code=0),
            template=template_def,
            actual_params=params)
    except ValidationError as e:
        return get_custom_fault_result(e.code, e.details)

    return get_content_correct_result()



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc
from collections import namedtuple
from sympy.logic.boolalg import And
from sympy.logic.boolalg import Not
from sympy.logic.boolalg import Or
from sympy.logic.boolalg import to_dnf as sympy_to_dnf
from sympy import Symbol


ConditionVar = namedtuple('ConditionVar', ['symbol_name', 'positive'])


class SymbolResolver(object):
    @abc.abstractmethod
    def is_relationship(self, symbol):
        pass

    @abc.abstractmethod
    def get_relationship_source_id(self, relationship):
        pass

    @abc.abstractmethod
    def get_relationship_target_id(self, relationship):
        pass

    @abc.abstractmethod
    def get_entity_id(self, entity):
        pass


def get_condition_common_targets(condition,
                                 definitions_index,
                                 symbol_resolver):
    """Return the targets that are common to all clauses of the condition.

    Common targets include:
       * And condition - any vertex that is part of the condition can
                         be a target
       * Not condition - no vertex that is part of the condition can
                         be a target
       * Or condition - vertices that appear in any "positive" part (i.e. one
                        that doesn't have a 'not' in front of it) of the
                        Or condition

    A complete description of all options can be found in Vitrage
    'external-actions' spec.

    The condition format:
        [[and_var1, and_var2, ...], or_list_2, ...]

    :return: A set of vertices that are common to all clauses of the condition
    """

    clauses_targets = []

    for clause in condition:
        clause_targets = set()

        for term in clause:
            if term.positive:
                symbol = definitions_index.get(term.symbol_name)
                if symbol and symbol_resolver.is_relationship(symbol):
                    clause_targets.add(
                        symbol_resolver.get_relationship_source_id(symbol))
                    clause_targets.add(
                        symbol_resolver.get_relationship_target_id(symbol))
                elif symbol:
                    clause_targets.add(symbol_resolver.get_entity_id(symbol))

        clauses_targets.append(clause_targets)

    return set.intersection(*clauses_targets)


def is_condition_include_positive_clause(condition):
    """Check if a condition is positive

    A positive condition has at least one part that is not 'not'

    Positive conditions:
        host_contains_instance
        host and not host_contains_instance

    Negative conditions:
        not host_contains_instance
        not host_contains_instance or not alarm_on_host

    The condition format:
        [[and_var1, and_var2, ...], or_list_2, ...]

    :return: True if the condition is positive
    """
    is_positive = False

    for clause in condition:
        for term in clause:
            if term.positive:
                is_positive = True

    return is_positive


def parse_condition(condition_str):
    """Parse condition string into an object

    The condition string will be converted here into DNF (Disjunctive
    Normal Form), e.g., (X and Y) or (X and Z) or (X and V and not W)
    ... where X, Y, Z, V, W are either entities or relationships
    more details: https://en.wikipedia.org/wiki/Disjunctive_normal_form

    The condition variable lists is then extracted from the DNF object.
    It is a list of lists. Each inner list represents an AND expression
    compound condition variables. The outer list presents the OR
    expression

      [[and_var1, and_var2, ...], or_list_2, ...]

    :param condition_str: the string as it written in the template
    :return: condition_vars_lists
    """

    condition_dnf = convert_to_dnf_format(condition_str)

    if isinstance(condition_dnf, Or):
        return extract_or_condition(condition_dnf)

    if isinstance(condition_dnf, And):
        return [extract_and_condition(condition_dnf)]

    if isinstance(condition_dnf, Not):
        return [(extract_not_condition_var(condition_dnf))]

    if isinstance(condition_dnf, Symbol):
        return [[(extract_condition_var(condition_dnf, True))]]


def convert_to_dnf_format(condition_str):
    condition_str = condition_str.replace(' and ', '&')
    condition_str = condition_str.replace(' AND ', '&')
    condition_str = condition_str.replace(' or ', '|')
    condition_str = condition_str.replace(' OR ', '|')
    condition_str = condition_str.replace(' not ', '~')
    condition_str = condition_str.replace(' NOT ', '~')
    condition_str = condition_str.replace('not ', '~')
    condition_str = condition_str.replace('NOT ', '~')

    return sympy_to_dnf(condition_str)


def extract_or_condition(or_condition):

    vars_ = []
    for var in or_condition.args:

        if isinstance(var, And):
            vars_.append(extract_and_condition(var))
        else:
            is_symbol = isinstance(var, Symbol)
            vars_.append([extract_condition_var(var, is_symbol)])

    return vars_


def extract_and_condition(and_condition):
    return [extract_condition_var(arg, isinstance(arg, Symbol))
            for arg in and_condition.args]


def extract_not_condition_var(not_condition):
    return [extract_condition_var(arg, False)
            for arg in not_condition.args]


def extract_condition_var(symbol, positive):
    if isinstance(symbol, Not):
        return extract_not_condition_var(symbol)[0]
    return ConditionVar(symbol.name, positive)



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json
import jwt
import os
import requests
import urllib

from cachetools import cached
from cachetools import LRUCache
from jwt.algorithms import RSAAlgorithm
from oslo_config import cfg
from oslo_log import log as logging
from oslo_middleware import base
from oslo_serialization import jsonutils
from pecan.core import abort
from webob import exc


LOG = logging.getLogger(__name__)


KEYCLOAK_GROUP = 'keycloak'
KEYCLOAK_OPTS = [
    cfg.StrOpt('auth_url', default='http://127.0.0.1:9080/auth',
               help='Keycloak authentication server ip',),
    cfg.StrOpt('insecure', default=False,
               help='If True, SSL/TLS certificate verification is disabled'),
    cfg.StrOpt('certfile',
               help='Required if identity server requires client certificate'),
    cfg.StrOpt('keyfile',
               help='Required if identity server requires client certificate'),
    cfg.StrOpt('cafile',
               help='A PEM encoded Certificate Authority to use when verifying'
               ' HTTPs connections. Defaults to system CAs.'),
    cfg.StrOpt(
        'user_info_endpoint_url',
        default='/realms/%s/protocol/openid-connect/userinfo',
        help='Endpoint against which authorization will be performed'
    ),
    cfg.StrOpt(
        'public_cert_url',
        default="/realms/%s/protocol/openid-connect/certs",
        help="URL to get the public key for particular realm"
        ),
    cfg.StrOpt(
        'keycloak_iss',
        help="keycloak issuer(iss) url "
             "Example: https://ip_add:port/auth/realms/%s"
    )
]


class KeycloakAuth(base.ConfigurableMiddleware):

    def __init__(self, application, conf=None):
        super(KeycloakAuth, self).__init__(application, conf)

        self.oslo_conf.register_opts(KEYCLOAK_OPTS, '%s' % KEYCLOAK_GROUP)
        self.auth_url = self._conf_get('auth_url', KEYCLOAK_GROUP)
        self.insecure = self._conf_get('insecure', KEYCLOAK_GROUP)
        self.certfile = self._conf_get('certfile', KEYCLOAK_GROUP)
        self.keyfile = self._conf_get('keyfile', KEYCLOAK_GROUP)
        self.cafile = self._conf_get('cafile', KEYCLOAK_GROUP) or \
            self._get_system_ca_file()
        self.user_info_endpoint_url = self._conf_get('user_info_endpoint_url',
                                                     KEYCLOAK_GROUP)
        self.public_cert_url = self._conf_get('public_cert_url',
                                              KEYCLOAK_GROUP)
        self.keycloak_iss = self._conf_get('keycloak_iss', KEYCLOAK_GROUP)

    @property
    def reject_auth_headers(self):
        header_val = 'Keycloak uri=\'%s\'' % self.auth_url
        return [('WWW-Authenticate', header_val)]

    @staticmethod
    def roles(decoded):
        return ','.join(decoded['realm_access']['roles']) \
            if 'realm_access' in decoded else ''

    @staticmethod
    def realm_name(decoded):
        # Get user realm from parsed token
        # Format is "iss": "http://<host>:<port>/auth/realms/<realm_name>",
        __, __, realm_name = decoded['iss'].strip().rpartition('/realms/')
        return realm_name

    def process_request(self, req):
        self._authenticate(req)

    def _authenticate(self, req):
        decoded = {}
        token = req.headers.get('X-Auth-Token')
        if token:
            decoded = self._decode(token)
        else:
            message = 'Auth token must be provided in "X-Auth-Token" header.'
            self._unauthorized(message)

        self.call_keycloak(token, decoded, decoded.get('aud'))

        self._set_req_headers(req, decoded)

    def _decode(self, token):
        try:
            return jwt.decode(token, algorithms=['RS256'], verify=False)
        except jwt.DecodeError:
            message = "Token can't be decoded because of wrong format."
            self._unauthorized(message)

    def call_keycloak(self, token, decoded, audience):
        if self.user_info_endpoint_url.startswith(('http://', 'https://')):
            endpoint = self.user_info_endpoint_url
            self.send_request_to_auth_server(endpoint, token)
        else:
            public_key = self.get_public_key(self.realm_name(decoded))
            try:
                if self.keycloak_iss:
                    self.keycloak_iss = self.keycloak_iss % \
                        self.realm_name(decoded)
                jwt.decode(token, public_key, audience=audience,
                           issuer=self.keycloak_iss, algorithms=['RS256'],
                           verify=True)
            except Exception:
                message = 'Token validation failure'
                self._unauthorized(message)

    def send_request_to_auth_server(self, endpoint, token=None):
        headers = {}
        if token:
            headers = {'Authorization': 'Bearer %s' % token}
        verify = None
        if urllib.parse.urlparse(endpoint).scheme == "https":
            verify = False if self.insecure else self.cafile
        cert = (self.certfile, self.keyfile) \
            if self.certfile and self.keyfile else None
        resp = requests.get(endpoint, headers=headers,
                            verify=verify, cert=cert)
        if not resp.ok:
            abort(resp.status_code, resp.reason)
        return resp.json()

    def _set_req_headers(self, req, decoded):
        req.headers['X-Identity-Status'] = 'Confirmed'
        req.headers['X-Roles'] = self.roles(decoded)
        req.headers["X-Project-Id"] = self.realm_name(decoded)

    def _unauthorized(self, message):
        body = {'error': {
            'code': 401,
            'title': 'Unauthorized',
            'message': message,
        }}

        raise exc.HTTPUnauthorized(body=jsonutils.dumps(body),
                                   headers=self.reject_auth_headers,
                                   charset='UTF-8',
                                   content_type='application/json')

    @staticmethod
    def _get_system_ca_file():
        """Return path to system default CA file."""
        # Standard CA file locations for Debian/Ubuntu, RedHat/Fedora,
        # Suse, FreeBSD/OpenBSD, MacOSX, and the bundled ca
        ca_path = ['/etc/ssl/certs/ca-certificates.crt',
                   '/etc/pki/tls/certs/ca-bundle.crt',
                   '/etc/ssl/ca-bundle.pem',
                   '/etc/ssl/cert.pem',
                   '/System/Library/OpenSSL/certs/cacert.pem',
                   requests.certs.where()]
        for ca in ca_path:
            LOG.debug("Looking for ca file %s", ca)
            if os.path.exists(ca):
                LOG.debug("Using ca file %s", ca)
                return ca
        LOG.warning("System ca file could not be found.")

    @cached(LRUCache(maxsize=32))
    def get_public_key(self, realm_name):
        keycloak_key_url = self.auth_url + self.public_cert_url % realm_name
        response_json = self.send_request_to_auth_server(keycloak_key_url)
        public_key = RSAAlgorithm.from_jwk(
            json.dumps(response_json["keys"][0]))
        return public_key


filter_factory = KeycloakAuth.factory
